;*******************************************************************************
;
;PROGRAMME      : LectureLAI_2000
;AUTEUR         : F. MAIGNAN
;CREATION       : 03/10/2008
;COMPILATEUR    : IDL
;OBJECTIF       : Lecture fichiers MODIS Senegal
;*******************************************************************************
;+
;
; .. _LectureLAIHDF.pro:
;
; ==================
; LectureLAIHDF.pro
; ==================
;
; Read LAI HDF files of a given year and write :file:`${PROJECT_OD}/LAI/laisen{yyyy}_float.txt` containing LAI in Senegal box.
;
; Format of ouput file have beeen choosen to make it readable by Qgis.
;
;    :param iodirin: based directory of HDF files to be read (before 
;		     :file:`LAI_{yyyy}`)
;
;    :type iodirin: string 
;    :raise iodirin: required
;
;    :param yyyy: year to be read
;    :type yyyy: integer
;    :raise yyyy: required
;
; SEE ALSO
; ========
;
; :func:`lecturelai`
;
; :func:`read_lai_2d`
;
; :func:`report <saxo:report>`
;
; :ref:`guide data LAI <data_lai>`
;
; EXAMPLES
; ========
;
; To generate :file:`${PROJECT_OD}/laisen2001_float.txt`::
;
;    .compile LectureLAIHDF
;    yyyy=2001L
;    iodirin='/net/zeus/usr/zeus/incas/mousson/LAI/'
;    lecturelaihdf, iodirin, yyyy
;
; TODO
; ====
;
; exit with error status if wrong parameter
;
; suppress iodirin parameter - should be $PROJECT_ID
;
; replace PRO by FUNCTION
;
; reactivate plot keyword
;
; externalize lecturelai function
;
; EVOLUTIONS
; ==========
;
; $Id: LectureLAIHDF.pro 335 2011-08-10 09:46:39Z pinsard $
;
; $URL$
;
; - fplod 20110422T155444Z aedon.locean-ipsl.upmc.fr (Darwin)
;
;   * suppress getenv
;
; - fplod 20101209T160741Z aedon.locean-ipsl.upmc.fr (Darwin)
;
;   * replace annee by yyyy
;
; - fplod 20101206T092230Z aedon.locean-ipsl.upmc.fr (Darwin)
;
;   * complete description
;   * build format instead of hard coded 46 lai/line
;   * replace TAB by 4 blanks
;
; - fplod 20101203T144742Z aedon.locean-ipsl.upmc.fr (Darwin)
;
;   * add compile_opt idl2, strictarrsubs
;   * add examples
;   * check parameters
;   * add iodirin parameter
;   * make it work for fplod@zeus
;
; - lelod 20101201
;
;   * modification format
;   * restriction zone Senegal hors Casamance
;
; - fplod 20100826T132803Z aedon.locean-ipsl.upmc.fr (Darwin)
;
;   * minimal rest header
;
; - modifi par L.Eymard le 25/07/2010
;
;   * pour l'adapter & l'environnement local (gestion des entrees - sorties)
;     modification du formattage du fichier ASCII de sortie pour
;     ecriture d'une ligne pour chaque "printf" (et non de plein de
;     lignes de 80 colonnes...)
;
;-
;
PRO LectureLAIHDF, iodirin, yyyy
;
compile_opt idl2, strictarrsubs
;
@cm_project
;
usage = 'lecturelaihdf, iodirin, yyyy'
;
; check required and optional parameters
nparam = N_PARAMS()
IF (nparam LT 2) THEN BEGIN
   ras = report(['Incorrect number of arguments.' $
         + '!C' $
         + 'Usage : ' + usage])
   STOP
ENDIF
;
arg_type = size(yyyy,/type)
IF ((arg_type NE 2) AND (arg_type NE 3)) THEN BEGIN
   ras = report(['Incorrect arg type yyyy' $
         + '!C' $
         + 'Usage : ' + usage])
   STOP
ENDIF
;
arg_type = size(iodirin,/type)
IF (arg_type NE 7) THEN BEGIN
   ras = report(['Incorrect arg type iodirin' $
         + '!C' $
         + 'Usage : ' + usage])
   STOP
ENDIF
;
; test if ${PROJECT_OD} defined
;CASE project_od_env OF
;  ''  :  BEGIN
;   msg = 'eee : ${PROJECT_OD} is not defined'
;   ras = report(msg)
;   STOP
;         END
;  ELSE: BEGIN
;   msg = 'iii : ${PROJECT_OD} is ' + project_od_env
;   ras = report(msg)
;        END
; ENDCASE
;
; existence and protection of iodirin
IF (FILE_TEST(iodirin, /DIRECTORY, /EXECUTABLE, /READ) EQ 0) THEN BEGIN
    msg = 'eee : the directory' + iodirin  + ' is not accessible.'
    ras = report(msg)
    STOP
ENDIF
;
; check if output data will be possible
iodirout = isadirectory(project_od_env)
;
; existence and protection
IF (FILE_TEST(iodirout, /DIRECTORY, /WRITE) EQ 0) THEN BEGIN
   msg = 'eee : the directory' + iodirout  + ' was not found.'
   ras = report(msg)
   STOP
ENDIF
;
PLOT=0
;
FORWARD_FUNCTION LectureLAI
;
;*** constantes
an=strcompress(string(yyyy),/remove_all)

file=iodirin+'LAI_2000/MOD15A2.A2000057.h16v07.005.2006269043641.hdf'
print,file
XDim=1200
YDim=1200
h=16
v=07
NB_LIG_MODIS = 21600
;
;latmin=14+45./60 & latmax=16+05./60
;lonmin=-17+30./60 & lonmax=-15+45./60

;restriction zone Senegal hors Casamance
latmin=13.6
latmax=16.3
lonmin=-17.4
lonmax=-12.52
;
;*** declarations
lat=MAKE_ARRAY(XDIM,YDIM,/FLOAT,VALUE=!VALUES.F_NAN)
lon=MAKE_ARRAY(XDIM,YDIM,/FLOAT,VALUE=!VALUES.F_NAN)
;
;*** initialisation environnement graphique
device,decomposed=0,retain=2
loadct,39
tvlct,r,g,b,/GET
!P.COLOR=0
!P.BACKGROUND=255
;
;*** version -> HDF4
id=HDF_OPEN(file)
HDF_LIB_INFO, id, VERSION=VER
PRINT, 'The file ', file,' was created with : ', VER
HDF_CLOSE, id
;
;*** lecture du LAI
LAI=LectureLAI(file)
;
;*** extraction
;*** attention, on compte lig/col a partir de 0
FOR ilig_MODIS=YDIM*v, YDIM*(v+1)-1 DO BEGIN
    lat[*,ilig_MODIS-YDIM*v] = 90.D - (ilig_MODIS + 0.5D)*180./NB_LIG_MODIS
    FOR jcol_MODIS=XDIM*h, XDIM*(h+1)-1 DO BEGIN
        Ni_MODIS = ROUND(NB_LIG_MODIS*cos(lat[jcol_MODIS-XDIM*h,ilig_MODIS-YDIM*v]*!DTOR))
        lon[jcol_MODIS-XDIM*h,ilig_MODIS-YDIM*v] = 180.D/Ni_MODIS*(jcol_MODIS - NB_LIG_MODIS + 0.5D)
    ENDFOR
ENDFOR
COND_LATLON=WHERE(lat GE latmin and lat LE latmax AND lon GE lonmin and lon LE lonmax,COMPLEMENT=COND_NOTOK)
zz=ARRAY_INDICES(lat,COND_LATLON)
indice_latmax=min(zz[1,*],max=indice_latmin)
indice_lonmin=min(zz[0,*],max=indice_lonmax)
lat_extract=lat[indice_lonmin:indice_lonmax,indice_latmax:indice_latmin]
lon_extract=lon[indice_lonmin:indice_lonmax,indice_latmax:indice_latmin]
lai[COND_NOTOK]=!VALUES.F_NAN
lai_extract=lai[indice_lonmin:indice_lonmax,indice_latmax:indice_latmin]
taille=size(lai_extract)
;print,'dimensions de la zone extraite',taille(1),taille(2)

;*** affichage graphique
IF KEYWORD_SET(plot) THEN BEGIN
    window,2,xsize=taille(1),ysize=taille(2)
    tvscl,lai_extract,/NAN,/ORDER, xsize=taille(1),ysize=taille(2)
ENDIF

path1=iodirin+'LAI_'+an

;*** boucle de lecture des fichiers LAI
;files=FILE_SEARCH(iodirin,'lai-200[0-4]/MOD15A2.A200????.h16v07.005.*.hdf',COUNT=nb_files)
files=FILE_SEARCH(path1,'*.hdf',COUNT=nb_files)
lai_extract=MAKE_ARRAY(nb_files,indice_lonmax-indice_lonmin+1,indice_latmin-indice_latmax+1,/FLOAT,VALUE=!VALUES.F_NAN)

datenum=intarr(nb_files)

FOR ifile=0, nb_files-1 DO BEGIN
    ;print,ifile, files[ifile]

    nom_court=FILE_BASENAME(files[ifile])
    res=STRSPLIT(nom_court,'.',/EXTRACT)
        date=strmid(res[1],5,3)
        ;print, date
        datenum[ifile]=fix(date)
    ;stop

;    date=strmid(files[ifile],68,3)
;    print, date
;    datenum(ifile)=fix(date)

    lai=LectureLAI(files[ifile], PLOT=plot)
    lai[COND_NOTOK]=-999.   ;!VALUES.F_NAN
    lai_extract[ifile,*,*]=lai[indice_lonmin:indice_lonmax,indice_latmax:indice_latmin]

;*** affichage graphique
;    IF KEYWORD_SET(plot) THEN BEGIN
;        window,2,XSIZE=taille(1),YSIZE=taille(2),title=STRING(ifile)
;        tvscl,lai_extract[ifile,*,*],/ORDER,/NAN
;        wait,0.5
;        ;stop
;    ENDIF
ENDFOR

IF KEYWORD_SET(plot) THEN BEGIN
    window,1
    FOR ilig=0, taille[2]-1 DO BEGIN
        FOR jcol=0, taille[2]-1 DO BEGIN
            IF FINITE(MAX(lai_extract[*,jcol,ilig],/NAN)) THEN BEGIN
                plot, datenum,lai_extract[*,jcol,ilig]
                wait,0.5
            ENDIF
        ENDFOR
    ENDFOR
ENDIF

;*** enregistrement ascii
fileascii=iodirout+'LAI/'+'laisen'+an+'_num.txt'
OPENW,lun,fileascii,/GET_LUN, ERROR = error
IF (error NE 0) then begin
   ras = report(['eee : can not open for writing '$
         + '!C' $
         + fileascii])
   STOP
ENDIF

print,fileascii
;printf,lun,datenum

; build format string
format_header='("Y;X"'
format_data='(f6.3,";",f7.3'
FOR idoy=1, nb_files DO BEGIN
    format_header = format_header + ",';',i3.3"
    format_data = format_data + ",';',f4.2"
ENDFOR
format_header = format_header + ")"
format_data = format_data + ")"
printf,lun,datenum, format=format_header

FOR ilig=0,taille[2]-1 DO BEGIN
    FOR jcol=0, taille[1]-1 DO BEGIN
        printf, lun, lat_extract[jcol,ilig] $
                   , lon_extract[jcol,ilig] $
                   , lai_extract[*,jcol,ilig] $
                   , format=format_data
    ENDFOR
ENDFOR
CLOSE,lun
FREE_LUN,lun

;STOP

;OPENW,lun,file,/GET_LUN
;printf,lun,datenum
;printf,lun,lai_extract
;CLOSE,lun
;FREE_LUN,lun

;STOP

END
;
;+
;
; ==========
; lecturelai
; ==========
;
; .. function: lai=lecturelai(file,/plot)
;
;    Read LAI HDF file
;
;    :param file: filename to be read
;    :type file: string
;    :raise file: required
;
;    :returns: lai or -1 if error
;    :rtype: 2D array +todo+ details
;
; SEE ALSO
; ========
;
; :ref:`guide data LAI <data_lai>`
;
; EXAMPLES
; ========
;
; ::
;
;    .compile LectureLAIHDF
;    file='/net/zeus/usr/zeus/incas/mousson/LAI/LAI_2000/MOD15A2.A2000057.h16v07.005.2006269043641.hdf'
;    lai=lecturelai(file,plot=0)
;
; :uses:
;
; func:`report <saxo:report>`
;
; :todo:
;
; :History:
;
; - fplod 20101206T083353Z aedon.locean-ipsl.upmc.fr (Darwin)
;
;   * add header
;   * add compilopt
;   * check parameter and keyword
;
;-
FUNCTION LectureLAI, file, PLOT=plot
;
compile_opt idl2, strictarrsubs
;
usage='lai=lecturelai(file,plot=0)
;
nparam = N_PARAMS()
IF (nparam LT 1) THEN BEGIN
   ras = report(['Incorrect number of arguments.' $
         + '!C' $
         + 'Usage : ' + usage])
   RETURN,-1
ENDIF
;
; test if file is a string
IF size(file, /type) NE 7 THEN BEGIN
   ras = report(['Incorrect arg type file' $
         + '!C' $
         + 'Usage : ' + usage])
   RETURN, -1
ENDIF
;
IF keyword_set(plot) THEN BEGIN
   IF size(plot,/type) NE 2 THEN BEGIN
      ras = report(['Incorrect keyword type plot' $
            + '!C' $
            + 'Usage : ' + usage])
      RETURN, -1
   ENDIF
ENDIF
;
sd_id=HDF_OPEN(file,/read)
IF sd_id EQ -1 THEN BEGIN
    ALB=MAKE_ARRAY(1200,1200,/FLOAT,VALUE=!VALUES.F_NAN)
    print, 'bad file :',file
ENDIF ELSE BEGIN
    sd_id=HDF_SD_START(file,/read)
    sds_id=HDF_SD_SELECT(sd_id,1)
    HDF_SD_GETDATA,sds_id,LAI
    HDF_SD_GETINFO,sds_id,DIMS=dims,NAME=name,NDIMS=ndims,TYPE=type,UNIT=unit
    id_valid_range=HDF_SD_ATTRFIND(sds_id,'valid_range')
    HDF_SD_ATTRINFO,sds_id,id_valid_range,data=valid_range
    id_scale_factor=HDF_SD_ATTRFIND(sds_id,'scale_factor')
    HDF_SD_ATTRINFO,sds_id,id_scale_factor,data=scale_factor
    id_fill_value=HDF_SD_ATTRFIND(sds_id,'_FillValue')
    HDF_SD_ATTRINFO,sds_id,id_fill_value,data=fill_value
    HDF_SD_ENDACCESS, sds_id
;*** Fermeture du fichier HDF
    HDF_SD_END, sd_id

;*** decodage
    LAI=FLOAT(LAI)
    COND_NAN=WHERE(LAI EQ fill_value[0] OR LAI LT valid_range[0] OR LAI GT valid_range[1])
    IF COND_NAN[0] NE -1 THEN LAI[COND_NAN]=!VALUES.F_NAN
    ;stop
    LAI=LAI*scale_factor[0]
    ;stop
ENDELSE

;*** affichage graphique
IF KEYWORD_SET(plot) THEN BEGIN
    window,0,xsize=600,ysize=600
    tvscl,REBIN(LAI,600,600),/NAN,/ORDER
    ;stop
ENDIF
;sd_id=HDF_SD_START(file,/read)
;sds_id=HDF_SD_SELECT(sd_id,1)
;HDF_SD_GETDATA,sds_id,LAI
;HDF_SD_GETINFO,sds_id,DIMS=dims,NAME=name,NDIMS=ndims,TYPE=type,UNIT=unit
;;print,'name=',name
;id_valid_range=HDF_SD_ATTRFIND(sds_id,'valid_range')
;HDF_SD_ATTRINFO,sds_id,id_valid_range,data=valid_range
;id_scale_factor=HDF_SD_ATTRFIND(sds_id,'scale_factor')
;HDF_SD_ATTRINFO,sds_id,id_scale_factor,data=scale_factor
;id_fill_value=HDF_SD_ATTRFIND(sds_id,'_FillValue')
;HDF_SD_ATTRINFO,sds_id,id_fill_value,data=fill_value
;HDF_SD_ENDACCESS, sds_id
;;*** Fermeture du fichier HDF
;HDF_SD_END, sd_id

;*** decodage
;LAI=FLOAT(LAI)
;COND_NAN=WHERE(LAI EQ fill_value[0] OR LAI LT valid_range[0] OR LAI GT valid_range[1])
;IF COND_NAN[0] NE -1 THEN LAI[COND_NAN]=!VALUES.F_NAN
;stop
;LAI=LAI*scale_factor[0]

;*** affichage graphique
;IF KEYWORD_SET(plot) THEN BEGIN
    ;window,0,xsize=600,ysize=600
    ;tvscl,REBIN(LAI,600,600),/NAN,/ORDER
    ;stop
;ENDIF

RETURN,LAI

END
