#! /bin/sh
#+
#
# .. _cat_clim_olr.sh:
#
# .. program:: cat_clim_olr.sh
#
# ===============
# cat_clim_olr.sh
# ===============
#
# --------------------------
# build OLR climatology file
# --------------------------
#
# SYNOPSIS
# ========
#
# ``cat_clim_olr.sh [-f] [-yearmin yyyy] [-yearmax yyyy]``
#
# DESCRIPTION
# ===========
#
# .. option:: -f
# .. option:: -yearmin <yyyy>
# .. option:: -yearmax <yyyy>
#
# Build OLR climatology file
# using mean daily file by year and reduce geographical domain
# to [60  W, 50  E] [30  S, 45  N].
#
# If :option:`-f` option is set, previously output files will be overwritten.
#
# By default, climatology is computed using [2000,2007].
#
# Using :option:`-yearmin` and :option:`-yearmax` parameters, you can choose time span.
#
# ``cat_clim_olr.sh`` read :file:`${PROJECT_ID}/olr.day.mean_{yyyy}_ng.nc` (produced by :ref:`olr_split.sh`).
#
# ``cat_clim_olr.sh`` use :command:`ncks` (NCO) to split each year file in 2 parts,
# first between yyyy0101 and yyyy0228,  second between yyyy0301 and yyyy1231.
#
# 365d year files are build using  use :command:`ncrcat` (NCO).
#
# :command:`ncea` (NCO) is used to compute average from all 365d year files.
#
# ``cat_clim_olr.sh`` write
# :file:`${PROJECT_OD}/olr.day.mean_climato_ng.nc`.
#
# Log file is written on :file:`${PROJECT_LOG}/cat_clim_olr.log.{YYYYMMDDTHHMMSSZ}`.
#
#
# .. only:: man
#
#    Figure is visible on PDF and HTML documents only.
#
# .. only:: html or latex
#
#     .. graphviz::
#
#        digraph cat_clim_olr {
#           graph [
#           rankdir="TB",
#           ]
#
#           olr_t3 [shape=ellipse,fontname=Courier,label="${PROJECT_ID}/olr.day.mean_yyyy_ng.nc"];
#           
#           olr_t4 [shape=ellipse,fontname=Courier,label="${PROJECT_OD}/olr.day.mean.climato_ng.nc"];
#           
#           cat_clim_olr [shape=box,
#           fontname=Courier,
#           color=blue,
#           URL="http://forge.ipsl.jussieu.fr/varamma/browser/trunk/src/cat_clim_olr.sh"
#           label="${PROJECT}/src/cat_clim_olr.sh"];
#           
#           {olr_t3} -> {cat_clim_olr} -> {olr_t4};
#        }
#           
# EXAMPLES
# ========
#
# To build OLR climatology using years [2000,2007] ::
#
#  $ cat_clim_olr.sh
#
#
# To see log file::
#
#  $ tlogd.sh cat_clim_olr
#
# To build OLR climatology using year 2000 only (silly for scientific work
# but usefull for test)::
#
#  $ cat_clim_olr.sh -yearmin 2000 -yearmax 2000
#
# COMMENTS
# ========
#
# If yearmin and yearmax are the same one can compare input and output files
# like this:
#
# For yearmin=yearmax=2000::
#
#  $ cdo diffv ${PROJECT_ID}/olr.day.mean_2000_ng.nc \
#              ${PROJECT_OD}/olr.day.mean_climato_ng.nc
#
#
# There must be a difference because 2000 is a leap year.
#
# For yearmin=yearmax=2001::
#
#   $ cdo diffv ${PROJECT_ID}/olr.day.mean_2001_ng.nc \
#               ${PROJECT_OD}/olr.day.mean_climato_ng.nc
#
# There must be no difference on values of olr and info fields ++  vrifier
#
# SEE ALSO
# ========
#
# :ref:`varamma_profile.sh`
#
# :ref:`olr_split.sh`
#
# :ref:`is_leap_year.sh`
#
# NCO_
#
# .. _NCO: http://nco.sourceforge.net/
#
# TODO
# ====
#
# .. todo::
#
# ++ unset
#
# ++ option debug/verbose
#
# ++ vrification accs filein
#
# can't compare
# ${PROJECT_ID}/olr.day.mean_2000_ng.nc and
# /usr/temp/fplod/varamma_d/olr.day.mean_climato_ng.nc_short with cdo diffv :
# ``cdo diffv (Abort): Grid size of the input fields do not match!``
#
#  "02-28 00:00:00.0 or 23:59
#
# !! pb zeus et dedale signal  reseau le 200807
#
# EVOLUTIONS
# ==========
#
# $Id$
#
# - fplod 20100702T075823Z aedon.locean-ipsl.upmc.fr (Darwin)
#
#   * add schema ((from varamma/notes/varamma_etape3.dot)
#
# - fplod 20100318T075106Z aedon.locean-ipsl.upmc.fr (Darwin)
#
#   * unset
#
# - fplod 2009-04-09T09:25:51Z aedon.locean-ipsl.upmc.fr (Darwin)
#
#   * process experimental output file to recover short olr with
#     add_offset and scale_factor (ncap)
#
# - fplod 2009-04-09T09:03:33Z aedon.locean-ipsl.upmc.fr (Darwin)
#
#   * add details on process (NCO commands and intermidiate files) in header
#   * remove intermidiate files
#   * choose ncea (versus ncra)
#
# - fplod 2009-04-08T15:55:02Z aedon.locean-ipsl.upmc.fr (Darwin)
#
#   * add details on input and output files
#
# - pinsard 2009-04-01T14:11:37Z camelot.ipsl.polytechnique.fr (Linux)
#
#   * add -yearmin and -yearmax parameters
#   * fix output file name
#   * to be consistent with overriding on NCO commands, add -f option
#
# - pinsard 2009-04-01T12:27:43Z camelot.ipsl.polytechnique.fr (Linux)
#
#   * force LANG=POSIX
#   * remove debug features
#   * force overriding on NCO commands to avoid interactive behaviour
#   * write log in logfile
#
# - fplod 2008-08-05T09:37:14Z aedon.locean-ipsl.upmc.fr (Darwin)
#
#   * creation from /homedata/eymard/scripts/script_cat_clim_AMSU
#
#-
#
# main program
system=$(uname)
case "${system}" in
   AIX|IRIX64)
      echo " www : no specific posix checking"
   ;;
   *)
      set -o posix
   ;;
esac
unset system
#
set -u
#
LANG=POSIX
#
command=$(basename ${0})
log_date=$(date -u +"%Y%m%dT%H%M%SZ")
#
usage=" Usage : ${command} [-yearmin yyyy] [-yearmax yyyy] [-f]"
#
# ++ test si ncks available
#
# default
force=0
yearmin=2000
yearmax=2007
#
minargcount=0
if [ ${#} -lt ${minargcount} ]
then
   echo "eee : not enought arguments"
   echo "${usage}"
   exit 1
fi
unset minargcount
#
while [ ${#} -gt 0 ]
do
   case ${1} in
      -yearmin)
         yearmin=${2}
         shift
      ;;
      -yearmax)
         yearmax=${2}
         shift
      ;;
      -f)
         force=1
      ;;
      -h)
         echo "${usage}"
         exit 0
      ;;
      *)
         # other choice
         echo "eee : unknown option ${1}"
         echo "${usage}"
         exit 1
      ;;
   esac
   # next flag
   shift
done
unset usage
#
# check for ${PROJECT_LOG} definition
if [ "${PROJECT_LOG}" = "" ]
then
   echo " eee : \${PROJECT_LOG} not defined"
   exit 1
fi
#
# check for ${PROJECT_LOG} existence
if [ ! -d ${PROJECT_LOG} ]
then
   echo " eee : ${PROJECT_LOG} not found"
   exit 1
fi
#
# check for permission access on PROJECT_LOG
if [ ! -x ${PROJECT_LOG} ]
then
   echo " eee : ${PROJECT_LOG} not reachable"
   exit 1
fi
#
# check for write permission on PROJECT_LOG
if [ ! -w ${PROJECT_LOG} ]
then
   echo " eee : ${PROJECT_LOG} not writable"
   exit 1
fi
#
log=${PROJECT_LOG}/$(basename ${0} .sh).log.${log_date}
echo "[Context]" 1>> ${log}
echo "command=$(basename ${0})" 1>>${log}
echo "runtime=${log_date}" 1>> ${log}
echo "log=${log}" 1>> ${log}
echo "" 1>> ${log}
echo "[Parameters]" 1>> ${log}
echo "yearmin=${yearmin}" 1>> ${log}
echo "yearmax=${yearmax}" 1>> ${log}
#
# check for input files
listin=""
iyear=${yearmin}
while [ ${iyear} -le ${yearmax} ]
do
   cyear=$(printf "%4.4d" ${iyear})
   filein=${PROJECT_ID}/olr.day.mean_${cyear}_ng.nc
   if [ ! -r ${PROJECT_ID}/olr.day.mean.nc ]
   then
      echo "eee : ${filein} not accessible" 1>>${log}
      exit 1
   fi
   file365[${iyear}]=${PROJECT_OD}/olr.day.mean_${cyear}_365d_ng.nc
   file365_1=${PROJECT_OD}/olr.day.mean_${cyear}_365d1_ng.nc
   file365_2=${PROJECT_OD}/olr.day.mean_${cyear}_365d2_ng.nc
   # get rid of yyyy0229
   is_leap_year.sh ${iyear}
   ily_status=${?}
   if [ ${ily_status} -eq 0 ]
   then
      time1=$(echo ${iyear} | awk ' {printf "%4.4d-01-01 00:00:00.0",$1}')
      time2=$(echo ${iyear} | awk ' {printf "%4.4d-02-28 00:00:00.0",$1}') #++23:59
      time3=$(echo ${iyear} | awk ' {printf "%4.4d-03-01 00:00:00.0",$1}')
      time4=$(echo ${iyear} | awk ' {printf "%4.4d-12-31 00:00:00.0",$1}') #++23:59
      # cut first part of year
      ncks \
         -O \
         --dimension time,"${time1}","${time2}" \
         ${filein} ${file365_1}
      status_ncks=${?}
      if [ ${status_ncks} -ne 0 ]
      then
         echo "eee : pb with ncks ${filein} between yyyy0101 and yyyy0228" 1>>${log}
         exit 1
      else
         echo "iii : ncks ${filein} between yyyy0101 and yyyy0228" 1>>${log}
      fi
      # cut second part of year
      ncks \
         -O \
         --dimension time,"${time3}","${time4}" \
         ${filein} ${file365_2}
      status_ncks=${?}
      if [ ${status_ncks} -ne 0 ]
      then
         echo "eee : pb with ncks ${filein} between yyyy0301 and yyyy1231" 1>>${log}
         exit 1
      else
         echo "iii : ncks ${filein} between yyyy0301 and yyyy1231" 1>>${log}
      fi
      # join first and second part
      ncrcat -O ${file365_1} ${file365_2} ${file365[${iyear}]}
      status_ncrcat=${?}
      if [ ${status_ncrcat} -ne 0 ]
      then
         echo "eee : pb with ncrcat ${filein}" 1>>${log}
         exit 1
      else
         echo "iii : ncrcat ${file365_1} ${file365_2}" 1>>${log}
         rm ${file365_1} ${file365_2}
      fi
   else
      cp ${filein} ${file365[${iyear}]}
      echo "iii : copy ${filein} ${file365[${iyear}]}" 1>>${log}
   fi
   listin="${listin} ${file365[${iyear}]}"
   iyear=$(( ${iyear} + 1 ))
done
#
# build fileout
# based on the first file of the serie (ie yearmin)
fileout=${PROJECT_OD}/$(basename ${file365[${yearmin}]} ${yearmin}_365d_ng.nc)climato_ng.nc
if [ -f ${fileout} ]
then
   if [ ${force} -eq 0 ]
   then
      echo "eee : ${fileout} already exists" 1>>${log}
      exit 1
   fi
fi
#
echo "iii : list of yearly files : ${listin}" 1>> ${log}
#
ncea -O ${listin} ${fileout}
status_ncea=${?}
if [ ${status_ncea} -ne 0 ]
then
   echo "eee : pb with ncea" 1>>${log}
   exit 1
else
   echo "iii : results in ${fileout}" 1>> ${log}
   rm ${listin}
fi
unset status_ncea
#
# conversion float to short with original scale factor and add_offset
ncap -O \
   -s "olr=short(olr);olr@add_offset=327.65f;olr@scale_factor=0.01f" \
   ${fileout} \
   ${fileout}_short
status_ncap=${?}
if [ ${status_ncap} -ne 0 ]
then
   echo "eee : pb with ncap" 1>>${log}
   exit 1
else
   echo "iii : results in ${fileout}_short" 1>> ${log}
fi
unset status_ncap
#
# end
unset log
set
exit 0
