function [Annee, Mois, Jour] = compar_tbseuil_image(Annee, Mois, Jour, lonmin, lonmax, latmin, latmax, Tb_SeuilSup_ini, dTb_SeuilSup, Tb_LimNoyau, Tps_SeuilInf)

%COMPAR_TBSEUIL_IMAGE ouvre des figures permettant de comparer les images
%de prcipitations cumules estimes  partir d'EPSAT avec diffrentes
%intensits de l'activit nuageuse estimes  partir de MSG, tout en
%faisant varier le seuil de temprature de brillance dliminant les nuages
%convectifs.

%
%+
%
% ======================
% compar_tbseuil_image.m
% ======================
%
% .. function:: compar_tbseuil_image(Annee, Mois, Jour, lonmin, lonmax, latmin, latmax, Tb_SeuilSup_ini, dTb_SeuilSup, Tb_LimNoyau, Tps_SeuilInf)
%
% DESCRIPTION
% ===========
%
% - Donnes d'entres :
%
%   * Annee (en caractres) : Anne (AAAA) du jour d'tude.
%   * Mois (en caractres) : Numro du mois (MM) du jour d'tude.
%   * Jour (en caractres) : Numro du Jour d'tude (JJ).
%
%   :param lonmin: longitude min, W < 0
%   :units lonmin: deg
%   :type lonmin: double
%   :raise lonmin: required
%
%   :param lonmax: longitude max, W < 0
%   :units lonmax: deg
%   :type lonmax: double
%   :raise lonmax: required
%
%   :param latmin: latitude min, N > 0
%   :units latmin: deg
%   :type latmin: double
%   :raise latmin: required
%
%   :param latmax: latitude max, N > 0
%   :units latmax: deg
%   :type latmax: double
%   :raise latmax: required
%
%   * Tps_SeuilInf : Critre de temps minimum de vie d'un nuage pour qu'il
%     soit pris en compte (en nombre de quarts d'heure, 3 par dfaut).
%   * Tb_SeuilSup_ini : Seuil de temprature de brillance maximale
%     dlimitant un nuage convectif. Valeur de dpart pour la comparaison.
%   * dTb_SeuilSup : Variation du seuil de temprature de brillance des
%     nuages. Cette variation est applique  4 reprises.
%   * Tb_LimNoyau : Seuil de temprature de brillance dlimitant le noyau
%     dans notre modle  deux couches.
%
% - Donnes de sorties :
%   * Annee (en caractres) : Anne (AAAA) du jour d'tude.
%   * Mois (en caractres) : Numro du mois (MM) du jour d'tude.
%   * Jour (en caractres) : Numro du Jour d'tude (JJ).
%
% Ouverture de figures permettant de comparer les images de prcipitations
% cumules estimes  partir d'EPSAT avec diffrentes intensits de
% l'activit nuageuse estimes  partir de MSG, tout en faisant varier le
% seuil de temprature de brillance dliminant les nuages convectifs.
%
% EXAMPLES
% ========
%
% Voir "demo".
%
% SEE ALSO
% ========
%
% :ref:`data_msg`
%
% :ref:`data_epsat`
%
% :func:`MSGread`
%
% :func:`intensityMSG_day`
%
% :func:`plot_intensityMSG`
%
% :func:`EPSATread`
%
% :func:`cumulEPSAT_day`
%
% :func:`plot_cumulEPSAT`
%
% TODO
% ====
%
% - fplod 20110609T071815Z aedon.locean-ipsl.upmc.fr (Darwin)
%
%   * pb crato octave ::
%
%        A(I,J): column index out of bounds; value 1 out of bound 1
%
% add demo with simul_epsat and simul_msg
%
% save figure
%
% EVOLUTIONS
% ==========
%
% $Id$
%
% $URL: svn+ssh://pinsard@forge.ipsl.jussieu.fr/ipsl/forge/projets/varamma/svn/trunk/src/compar_tbseuil_image.m $
%
% - fplod 20110830T074359Z cratos (Linux)
%
%   * suppress interactivity
%
% - pinsard 2011-06-08T15:31:13Z loholt1.ipsl.polytechnique.fr (Linux)
%
%   * add box parameter
%
% - fplod 20110309T165153Z aedon.locean-ipsl.upmc.fr (Darwin)
%
%   * suppress fontsize and fontname (known problem for octave)
%   * octave does not like [argout1,~, argout3]=myfunc() so replace ~ by Temps
%
% - fplod 20110304T090823Z aedon.locean-ipsl.upmc.fr (Darwin)
%
%   * fix rst header
%
% - jaclod 20110302
%
%   * Ajout de la doc et de la dmonstration
%
% - jaclod 20110226
%
%   * Cration
%
%-
usage='[Annee, Mois, Jour] = compar_tbseuil_image(Annee, Mois, Jour, lonmin, lonmax, latmin, latmax, Tb_SeuilSup_ini, dTb_SeuilSup, Tb_LimNoyau, Tps_SeuilInf)';
if nargin~=nargin(mfilename)
   disp(['Incorrect number of input arguments ' num2str(nargin)]);
   error(usage)
end
%
if nargout ~= nargout(mfilename)
   disp(['Incorrect number of output arguments ' num2str(nargout)]);
   error(usage);
end
%
%%%%%%%%%%%%%
% Paramtres
%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%
% Image du cumul EPSAT
%%%%%%%%%%%%%%%%%%%%%%%%

% On lit le fichier EPSAT correspondant  notre date. 
% On ne se sert pas du "Temps" mais si on ne le met pas, il y a un problme au niveau des variables de sorties).
[PluieEPSAT, Temps, Longitudes, Latitudes, Annee, Mois, Jour] = EPSATread(Annee, Mois, Jour, lonmin, lonmax, latmin, latmax);
clear Temps;

% On calcule le cumule des prcipitations.
[CumulEPSAT] = cumulEPSAT_day(PluieEPSAT);

% On trace l'image des prcipitations cumules dans une premire zone.
subplot(2,3,1);
plot_cumulEPSAT(CumulEPSAT, Annee, Mois, Jour, Longitudes, Latitudes);

% On modifie le titre.
title(['Precipitations cumules en mm pour le ',...
    ([Annee '-' Mois '-' Jour])])

%%%%%%%%%%%%%%%%%%%%%%%%


%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Images des intensits MSG
%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% On lit le fichier MSG correspondant  notre date.
[TbMSG, Temps, Longitudes, Latitudes, Annee, Mois, Jour] = MSGread(Annee, Mois, Jour, lonmin, lonmax, latmin, latmax);

% On boucle ensuite pour avoir diffrent paramtres et, pour chaque cas,
% calculer l'intentensit et tracer l'image.
for i=1:5;

    % On calcule l'intensit.
    Tb_SeuilSup = Tb_SeuilSup_ini + (i-1)*dTb_SeuilSup;
    [ij_intensity, Tps_SeuilInf, Tb_SeuilSup, Tb_LimNoyau] = intensityMSG_day(Temps, TbMSG, Tps_SeuilInf, Tb_SeuilSup, Tb_LimNoyau);

    % On trace l'image de l'intensit.
    subplot(2,3,i+1);
    plot_intensityMSG(ij_intensity, Annee, Mois, Jour, Longitudes, Latitudes);

    % On modifie le titre.
    title(['Intensits avec un seuil de ' num2str(Tb_SeuilSup) 'K'])

end

whos

end

%!demo
%! varamma_startup;
%! more off
%! Annee = '2006';
%! Mois = '07';
%! Jour = '27';
%! latmin = 10;
%! latmax = 20;
%! lonmin = -20;
%! lonmax = -10;
%! Tb_SeuilSup_ini = 210;
%! dTb_SeuilSup = 10;
%! Tb_LimNoyau = 190;
%! Tps_SeuilInf = 3;
%! [Annee, Mois, Jour] = compar_tbseuil_image(Annee, Mois, Jour, lonmin, lonmax, latmin, latmax, Tb_SeuilSup_ini, dTb_SeuilSup, Tb_LimNoyau, Tps_SeuilInf);
