#! /bin/sh
#+
#
# .. program:: create_nc.sh
#
# .. _create_nc.sh:
#
# ============
# create_nc.sh
# ============
#
# DESCRIPTION
# ===========
#
# Appel des scripts FERRET pour la cration de sorties NetCDF
#
# Liste des paramtres :
#
#  $1   year
#  $2   month
#  $3   day
#  $4   hour
#  $5   min
#  $6   Nom fichier d'entre (chemin complet)
#  $7   Nom fichier de definition(chemin complet)
#  $8   Nom fichier de sortie NetCDF (chemin complet)
#  $9   Lgende
#  $10   Units
#  $11   scale_factor
#  $12   add_offset
#  $13   format (i2, i3, ...)
#  $14   missing value
#  $15   swap
#  $16   lonmin
#  $17   lonmax
#  $18   latmin
#  $19   latmax
#
# .. only:: man
#
#    Figure is visible on PDF and HTML documents only.
#
# .. only:: html or latex
#
#     .. graphviz::
#
#        digraph create_nc {
#           graph [
#           rankdir="TB",
#           ]
#
#           msg_t3 [shape=ellipse,fontname=Courier,label="${PROJECT_ID}/MSG/yyyy/mm/dd/yyyymmddhhmn_msg-prod_map_15min.raw"];
#           msg_t4 [shape=ellipse,fontname=Courier,label="${PROJECT_ID}/MSG/yyyy/mm/dd/yyyymmddhhmn_msg-prod_map_15min.def"];
#
#           msg_t5 [shape=ellipse,fontname=Courier,label="${PROJECT_ID}/MSG/yyyy/mm/dd/yyyymmddhhmn_msg-prod_map_15min.nc"];
#
#           log [shape=ellipse,fontname=Courier,label="${PROJECT_LOG}/create_nc.log{YYYYMMDDTHHMMSSZ}"];
#
#           create_nc [shape=box,
#           fontname=Courier,
#           color=blue,
#           URL="http://forge.ipsl.jussieu.fr/varamma/browser/trunk/src/create_nc.sh",
#           label="${PROJECT}/src/create_nc.sh"];
#
#           {msg_t3 msg_t4} -> {create_nc} -> {log msg_t5}
#
#          }
#
# EXAMPLES
# ========
#
# You already used :ref:`get_msg.sh` to get MSG reference data of 20060801 and 
# you want to produce the NetCDF file::
#
#  $ create_nc.sh 2006 08 01 00 00 /homedata/pinsard/varamma_d/MSG/2006/08/01/200608010000_msg-tb108_map_15min.raw /homedata/pinsard/varamma_d/MSG/2006/08/01/200608010000_msg-tb108_map_15min.def /homedata/pinsard/varamma_d/MSG/2006/08/01/200608011000_msg-tb108_map_15min.nc 'Brightness Temperatures (10.8 microns)' K 0.01 327.68 2 32767 1 -25 25 -5 20
#
# And look at log file with ::
#
#  $ tlogd.sh create_nc
#
# and of course on files in ${PROJECT_ID}.
#
# SEE ALSO
# ========
#
# :ref:`guide data MSG <data_msg>`
#
# Called by : :ref:`traite_msg-prod.sh`
#
# Use:
# :ref:`create_nc.jnl`
# Ferret_
#
# .. _Ferret: http://ferret.pmel.noaa.gov/Ferret/
#
# NCO_
#
#  .. _NCO: http://nco.sourceforge.net/
#
# TODO
# ====
#
# .. todo::
#
# consolidation
#
# get rid of path in arguments
#
# add -f option
#
# check argument 
#
# complete usage
#
# netcdf CF convention
#
# EVOLUTIONS
# ==========
#
# $Id$
#
# - pinsard 2011-05-04T15:12:22Z loholt1.ipsl.polytechnique.fr (Linux)
# 
#   * log file
#
# - pinsard 2011-05-02T09:19:21Z loholt1.ipsl.polytechnique.fr
#
#   * suppress 0 in "exit 0".
#   * no more intermidiate run*.jnl
#   * reduce stdout
#
# - pinsard 2010-07-22T10:07:34Z loholt1.ipsl.polytechnique.fr (Linux)
#
#   * replace /opt/ferret-6.0.8/bin/ferret (not anymore available on climserv) by
#     ferret
#
# - fplod 20100608T143812Z aedon.locean-ipsl.upmc.fr (Darwin)
#
#   * minimal header
#
# - Karim Ramage ?
#
#-
system=$(uname)
case "${system}" in
   AIX|IRIX64)
     echo "www : no specific posix checking"
     date_cmd=date
   ;;
   Darwin)
     set -o posix
     date_cmd=gdate
   ;;
   Linux)
     set -o posix
     date_cmd=date
   ;;
   *)
    set -o posix
   ;;
esac
unset system
#
LANG=POSIX
#
command=$(basename ${0})
log_date=$(date -u +"%Y%m%dT%H%M%SZ")
#
usage=" Usage : ${command} ++"
#
hostname=$(hostname)
#
# default
force=1
#
# check parameters
#++
#
log=${PROJECT_LOG}/$(basename ${0} .sh).log.${log_date}
echo "[Context]" 1>> ${log}
echo "command=$(basename ${0})" 1>>${log}
echo "hostname=${hostname}" 1>> ${log}
echo "runtime=${log_date}" 1>> ${log}
echo "log=${log}" 1>> ${log}
unset log_date
echo "" 1>> ${log}
echo "[Parameters]" 1>> ${log}
echo "++" 1>> ${log}
echo "" 1>> ${log}
#
tool=ferret
type ${tool} 1> /dev/null 2>&1
status=${?}
if [ ${status} -ne 0 ]
then
   echo "${command} : eee : tool ${tool} not found"
   exit 1
fi
unset status
unset tool
#
tool=ncap
type ${tool} 1> /dev/null 2>&1
status=${?}
if [ ${status} -ne 0 ]
then
   echo "${command} : eee : tool ${tool} not found"
   exit 1
fi
unset status
unset tool
#
tool=ncatted
type ${tool} 1> /dev/null 2>&1
status=${?}
if [ ${status} -ne 0 ]
then
   echo "${command} : eee : tool ${tool} not found"
   exit 1
fi
unset status
unset tool
#
[ ${2} == "01" ]&&{ mois="jan"; }
[ ${2} == "02" ]&&{ mois="feb"; }
[ ${2} == "03" ]&&{ mois="mar"; }
[ ${2} == "04" ]&&{ mois="apr"; }
[ ${2} == "05" ]&&{ mois="may"; }
[ ${2} == "06" ]&&{ mois="jun"; }
[ ${2} == "07" ]&&{ mois="jul"; }
[ ${2} == "08" ]&&{ mois="aug"; }
[ ${2} == "09" ]&&{ mois="sep"; }
[ ${2} == "10" ]&&{ mois="oct"; }
[ ${2} == "11" ]&&{ mois="nov"; }
[ ${2} == "12" ]&&{ mois="dec"; }
#
if [ -f ${8} ]
then
  if [ ${force} -eq 0 ]
  then
     echo "eee : ${8} already exists" 1>> ${log}
     exit 1
  else
     rm -f ${8} 
  fi
fi
ferret -gif -nojnl -memsize 256 << EOF >> ${log} 2>&1
  CANCEL MODE VERIFY
  GO create_nc.jnl "${3}-${mois}-${1} ${4}:${5}:00" "${6}" "${7}" "${8}" "${9}" "${10}" "${13}" "${15}" "${16}" "${17}" "${18}" "${19}"
  quit
EOF
if [ ! -f ${8} ]; then
  echo "${8} non creation failed." >> ${log} 2>&1
  exit 1
fi
#
[ "${13}" == "1" ]&&{
    ncap -O -h -s "DATA=byte(DATA);" ${8} ${8} >> ${log} 2>&1
    ncatted -O -h -a missing_value,DATA,o,s,${14} ${8} >> ${log} 2>&1
    ncatted -O -h -a _FillValue,DATA,o,s,${14} ${8} >> ${log} 2>&1
}
[ "${13}" == "2" ]&&{
    ncap -O -h -s "DATA=short(DATA);" ${8} ${8}; >> ${log} 2>&1
    ncatted -O -h -a missing_value,DATA,o,s,${14} ${8} >> ${log} 2>&1
    ncatted -O -h -a _FillValue,DATA,o,s,${14} ${8} >> ${log} 2>&1
}
[ "${13}" == "4" ]&&{
    ncap -O -h -s "DATA=int(DATA);" ${8} ${8} >> ${log} 2>&1
    ncatted -O -h -a missing_value,DATA,o,l,${14} ${8}  >> ${log} 2>&1
    ncatted -O -h -a _FillValue,DATA,o,l,${14} ${8}  >> ${log} 2>&1
}
#
if [ "${9}" != 0 ]; then
  ncatted -O -h -a long_name,DATA,o,c,"${9}" ${8}  >> ${log} 2>&1
fi
if [ "${10}" != 0 ]; then
  ncatted -O -h -a units,DATA,o,c,"${10}" ${8}  >> ${log} 2>&1
fi
if [ "${11}" != 1 ]; then
  ncatted -O -h -a scale_factor,DATA,o,f,"${11}" ${8}  >> ${log} 2>&1
fi
if [ "${12}" != 0 ]; then
  ncatted -O -h -a add_offset,DATA,o,f,${12} ${8}  >> ${log} 2>&1
fi
#
echo "iii : ${8} ready"  >> ${log} 2>&1
#
exit
