function [lon_area, lat_area] = define_area(lon_value, lat_value, area_type, radius_value, dlon_value, dlat_value)

%DEFINE_AREA returns coordinates of points of polygon

%+
%
% =============
% define_area.m
% =============
%
% .. function:: define_area(lat_value, lon_value, area_type, radius_value, dlon_value, dlat_value)
%
% DESCRIPTION
% ===========
%
% Define points of polygon.
%
% If **area_type** is set to ``circle``, polygon is an approximated circle
% centered on **lon_value** and **lat_value** with **radius** as radius.
%
% If **area_type** is set to ``rectangle_center``, polygon is centered on
% **lon_value** and **lat_value** and height is **dlat_value**\*2, width is
% **dlon_value**\*2.
%
% If **area_type** is set to ``rectangle_corner``, polygon is defined
% by [lon_value(1),lon_value(2)] and [lat_value(1),lat_value(2)]
%
% All parameters are in decimal degrees.
%
% Around ++ N, 0.1 degree is approximately 10. km and 1/4 degree is approximately 25 km.
%
% Outputs (**lon_area** and **lat_area**) are organized to be used
% as 3d and 4th argument of :func:`inpolygon <octave:inpolygon>`.
%
%    :param lon_value: longitude of the center of the polygon , W < 0
%    :units lon_value: decimal degree
%    :type lon_value: double if **area_type** is set
%                     to ``rectangle_center`` or ````circle``
%                     1D array of 2 double if **area_type** is set
%                     to ``rectangle_corner``
%    :raise lon_value: required
%
%    :param lat_value: latitude of the center of the polygon, N > 0
%    :units lat_value: decimal degree
%    :type lat_value: double **area_type** is set
%                     to ``rectangle_center`` or ````circle``
%                     1D array of 2 double if **area_type** is set
%                     to ``rectangle_corner``
%
%    :raise lat_value: required
%
%    :param radius: radius of the approximated circle
%    :units radius: decimal degree
%    :type radius: double
%    :raise radius: required
%
%    :param dlon_value: semi width of the polygon
%    :units dlon_value: decimal degree
%    :type dlon_value: double
%    :raise dlon_value: required
%
%    :param dlat_value: semi height of the polygon
%    :units dlat_value: decimal degree
%    :type dlat_value: double
%    :raise dlat_value: required
%
%    :returns lon_area: longitude of each point of the polygon
%    :units lon_area: decimal degree
%    :type lon_area: 1d array of double
%
%    :returns lat_area: latitude of each point of the polygon
%    :units lat_area: decimal degree
%    :type lat_area: 1d array of double
%
% EXAMPLES
% ========
%
%  see demos
%
% SEE ALSO
% ========
%
% :func:`inpolygon <octave:inpolygon>`.
%
% :func:`reduce_domain`
%
% TODO
% ====
%
% check non NaN value is circle
%
% check non NaN value is rectangle
%
% check if correct order in outputs to used in inpolygon
%
% sphinx intermapping octave
%
% add tests
%
% check dimension of input parameters
%
% EVOLUTIONS
% ==========
%
% $Id$
% $URL: svn+ssh://pinsard@forge.ipsl.jussieu.fr/ipsl/forge/projets/varamma/svn/trunk/src/define_area.m $
%
% - fplod 20110311T142415Z aedon.locean-ipsl.upmc.fr (Darwin)
%
%   * replace rectangle area_type to rectangle_center
%   * add rectangle_corner option
%
% - fplod 20110224T115157Z aedon.locean-ipsl.upmc.fr (Darwin)
%
%   * add inpolygon usage in demo
%
% - fplod 20110224T112157Z aedon.locean-ipsl.upmc.fr (Darwin)
%
%   * compute circle coordinates
%
% - fplod 20110224T104319Z aedon.locean-ipsl.upmc.fr (Darwin)
%
%   * add plot in demos
%
% - fplod 20110223T162945Z aedon.locean-ipsl.upmc.fr (Darwin)
%
%   * creation
%
%-
usage='[lon_area, lat_area] = define_area(lon_value, lat_value, area_type, radius_value, dlon_value, dlat_value)';
%
if nargin~=nargin(mfilename)
   disp(['Incorrect number of input arguments ' num2str(nargin)]);
   error(usage);
end
%
if nargout ~= nargout(mfilename)
   disp(['Incorrect number of output arguments ' num2str(nargout)]);
   error(usage);
end
%
arg_info=whos('lon_value');
if ~strcmp(arg_info.class,'double')
   disp(['Incorrect type of arg lon_value ' arg_info.class]);
   whos lon_value
   error(usage);
end
%
%
arg_info=whos('lat_value');
if ~strcmp(arg_info.class,'double')
   disp(['Incorrect type of arg lat_value ' arg_info.class]);
   whos lat_value
   error(usage);
end
clear arg_info
%
arg_info=whos('radius_value');
if ~strcmp(arg_info.class,'double')
   disp(['Incorrect type of arg radius_value ' arg_info.class]);
   whos radius_value
   error(usage);
end
clear arg_info
%
arg_info=whos('dlon_value');
if ~strcmp(arg_info.class,'double')
   disp(['Incorrect type of arg dlon_value ' arg_info.class]);
   whos dlon_value
   error(usage);
end
%
arg_info=whos('dlat_value');
if ~strcmp(arg_info.class,'double')
   disp(['Incorrect type of arg dlat_value ' arg_info.class]);
   whos dlat_value
   error(usage);
end
clear arg_info
%
arg_info=whos('area_type');
if ~strcmp(arg_info.class,'char')
   disp(['Incorrect type of arg area_type ' arg_info.class]);
   whos area_type
   error(usage);
end
clear arg_info
%
switch area_type
   case {'circle'}
   case {'rectangle_center'}
   case {'rectangle_corner'}
   otherwise
      error(['eee : unknown area_type ' area_type])
end
%
% preallocation of outputs
% nb_point_max is set to 5 to define a rectangle (four points + 1 - the last point is equal to the first one)
% nb_point_max is set to 21 to define a circle
switch area_type
   case {'circle'}
      nb_point_max=21.;
   case {'rectangle_center','rectangle_corner'}
      nb_point_max=5;
   otherwise
      error(['eee : unknown area_type ' area_type])
end
lon_area=zeros(nb_point_max,1);
lon_area(:)=NaN;
lat_area=zeros(nb_point_max,1);
lat_area(:)=NaN;
%
switch area_type
   case {'circle'}
        theta = linspace(0,2*pi,nb_point_max-1);
        lon_area= cos(theta) * radius_value + lon_value;
        lat_area= sin(theta) * radius_value + lat_value;
        clear theta
   case {'rectangle_center'}
        lon_area(1)=lon_value - dlon_value;
        lat_area(1)=lat_value - dlat_value;
        lon_area(2)=lon_value + dlon_value;
        lat_area(2)=lat_value - dlat_value;
        lon_area(3)=lon_value + dlon_value;
        lat_area(3)=lat_value + dlat_value;
        lon_area(4)=lon_value - dlon_value;
        lat_area(4)=lat_value + dlat_value;
   case {'rectangle_corner'}
        lon_area(1)=lon_value(1);
        lat_area(1)=lat_value(1);
        lon_area(2)=lon_value(1);
        lat_area(2)=lat_value(2);
        lon_area(3)=lon_value(2);
        lat_area(3)=lat_value(2);
        lon_area(4)=lon_value(2);
        lat_area(4)=lat_value(1);
   otherwise
      error(['eee : unknown area_type ' area_type])
end
lon_area(end)=lon_area(1);
lat_area(end)=lat_area(1);
%
clear nb_point_max
%
clear usage
%
end

%!demo
%! % rectangle_center
%! lon_value=-20;
%! lat_value=10;
%! area_type='rectangle_center';
%! radius_value=NaN;
%! dlon_value=.25;
%! dlat_value=.50;
%! [lon_area, lat_area] = define_area(lon_value, lat_value, area_type, radius_value, dlon_value, dlat_value)
%! close all
%! % define axis not to close from area to be plot
%! ymin=lat_value - 2*dlat_value - 2;
%! ymax=lat_value + 2*dlat_value + 2;
%! xmin=lon_value - 2*dlon_value - 2;
%! xmax=lon_value + 2*dlon_value + 2;
%! plot(lon_area, lat_area,'-')
%! % build text with center coordinates and size info
%! center_label=geolocation_to_string(lon_value, lat_value);
%! size_label=[ num2str(dlon_value) ' ' num2str(dlat_value)];
%! label=['center ' center_label ' size ' size_label];
%! hold on
%! text(lon_value, lat_value, label)
%! % limit world map around xmin, xmax and xmin, xmax
%! axis([xmin xmax ymin ymax]);

%!demo
%! % rectangle_corner
%! lon_value=[-20 -10];
%! lat_value=[10 20];
%! area_type='rectangle_corner';
%! radius_value=NaN;
%! dlon_value=NaN;
%! dlat_value=NaN;
%! [lon_area, lat_area] = define_area(lon_value, lat_value, area_type, radius_value, dlon_value, dlat_value)
%! close all
%! % define axis not to close from area to be plot
%! ymin=lat_value(1) - 2;
%! ymax=lat_value(2) + 2;
%! xmin=lon_value(1) - 2;
%! xmax=lon_value(2) + 2;
%! plot(lon_area, lat_area,'-')
%! % build text with corners coordinates
%! corner_label=geolocation_to_string(lon_value(1), lat_value(1));
%! label=['corner 1 ' corner_label];
%! hold on
%! text(lon_value(1), lat_value(1), label)
%! corner_label=geolocation_to_string(lon_value(2), lat_value(2));
%! label=['corner 2 ' corner_label];
%! hold on
%! text(lon_value(2), lat_value(2), label)
%! % limit world map around xmin, xmax and xmin, xmax
%! axis([xmin xmax ymin ymax]);

%!demo
%! % circle
%! lon_value = -20;
%! lat_value = 10;
%! area_type = 'circle';
%! radius_value = .25;
%! dlon_value = NaN;
%! dlat_value = NaN;
%! [lon_area, lat_area] = define_area(lon_value, lat_value, area_type, radius_value, dlon_value, dlat_value)
%! close all
%! % define axis not to close from area to be plot
%! ymin = lat_value - 2*radius_value - 2;
%! ymax = lat_value + 2*radius_value + 2;
%! xmin = lon_value - 2*radius_value - 2;
%! xmax = lon_value + 2*radius_value + 2;
%! plot(lon_area, lat_area,'-')
%! % build text with center coordinates and size info
%! center_label = geolocation_to_string(lon_value, lat_value);
%! size_label = [ num2str(radius_value)];
%! label = ['center ' center_label ' size ' size_label];
%! hold on
%! text(lon_value, lat_value, label)
%! % limit world map around xmin, xmax and xmin, xmax
%! axis([xmin xmax ymin ymax]);
%! axis('square');

%!demo
%! % in/out polygon rectangle_center
%! lon_value = -20;
%! lat_value = 10;
%! area_type = 'rectangle_center';
%! radius_value = NaN;
%! dlon_value = .25;
%! dlat_value = .50;
%! [lon_area, lat_area] = define_area(lon_value, lat_value, area_type, radius_value, dlon_value, dlat_value);
%! in_area = inpolygon(lon_value, lat_value, lon_area, lat_area)
%! in_area = inpolygon(lon_value+2*dlon_value+1, lat_value, lon_area, lat_area)

%!demo
%! % in/out polygon rectangle_corner
%! lon_value = [-20 -10];
%! lat_value = [10 20];
%! area_type = 'rectangle_corner';
%! radius_value = NaN;
%! dlon_value = NaN;
%! dlat_value = NaN;
%! [lon_area, lat_area] = define_area(lon_value, lat_value, area_type, radius_value, dlon_value, dlat_value)
%! in_area = inpolygon(lon_value(1), lat_value(1), lon_area, lat_area)
%! in_area = inpolygon(lon_value(1)-1, lat_value(1)-1, lon_area, lat_area)

%!demo
%! % in/out polygon circle
%! lon_value = -20;
%! lat_value = 10;
%! area_type = 'circle';
%! radius_value = .25;
%! dlon_value = NaN;
%! dlat_value = NaN;
%! [lon_area, lat_area] = define_area(lon_value, lat_value, area_type, radius_value, dlon_value, dlat_value);
%! in_area = inpolygon(lon_value, lat_value, lon_area, lat_area)
%! in_area = inpolygon(lon_value+2*radius_value+1, lat_value, lon_area, lat_area)
