; docformat = 'rst'
;+
;
; .. _extract_amsuab.pro:
;
; ==================
; extract_amsuab.pro
; ==================
;
; prgm de lecture des fichiers AMSU (A et B ou B seulement)
;
; If use_amusa parameter is set to 1,
; read :file:`${PROJECT_ID}/list_filea` inside :ref:`traite_amsuab.sh`
;
; read :file:`${PROJECT_ID}/list_fileb` inside :ref:`traite_amsuab.sh`.
;
; decode les noms des fichiers dans la date choisie, puis
; appelle le prgm de lecture
;
; ecrit les donnees pour la zone (lat/long) choisie
;
; le fichier de sortie contient dans l'ordre mm, jj (jour de
; l'annee), le temps (identique pour les points d'une meme fauchee), le
; no satellite, le no de fov (1 a 40 ou 1 a 90), long, lat,
; tb1,... tbn
;
; Les valeurs manquantes sont codes par NaN.
;
; .. only:: man
;
;    Figure is visible on PDF and HTML documents only.
;
; .. only:: html or latex
;
;     .. graphviz::
;
;        digraph extract_amsu {
;
;           amsu_lista [shape=ellipse,fontname=Courier,label="${PROJECT_ID}/list_filea"];
;           amsu_listb [shape=ellipse,fontname=Courier,label="${PROJECT_ID}/list_fileb"];
;           amsu_t1 [shape=ellipse,fontname=Courier,label="${PROJECT_ID}/AMSU/AMSU*/L1C/yyyy/yyyy_mm_dd/*.L1C"];
;           amsu_t2 [shape=ellipse,fontname=Courier,label="${PROJECT_ID}/yyyy/mm/amsua_amsub_yyyymmdd_geomin_geomax.dat"];
;
;           extract_amsuab [shape=box,
;           fontname=Courier,
;           color=blue,
;           URL="http://forge.ipsl.jussieu.fr/varamma/browser/trunk/src/extract_amsuab.pro",
;           label="${PROJECT}/src/extract_amsuab.pro"];
;
;           {amsu_lista amsu_listb amsu_t1} -> {extract_amsuab} -> {amsu_t2};
;       }
;
; :Categories: AMSU
;
; :Params:
;   use_amusa
;   yyyy
;   mm
;   lonmin
;   lonmax
;   latmin
;   latmax
;
; :Examples:
;
;  Creating an articial list of AMSU-B and AMSU-A files::
;
;   $ echo "${PROJECT_ID}/AMSU/AMSUBN15/L1C/2006/2006_08_01/NSS.AMBX.NK.D06213.S0110.E0255.B4271112.GC.L1C" > ${PROJECT_ID}/list_fileb
;   $ echo "${PROJECT_ID}/AMSU/AMSUBN15/L1C/2006/2006_08_01/NSS.AMBX.NK.D06213.S0250.E0437.B4271213.GC.L1C" >> ${PROJECT_ID}/list_fileb
;   $ echo "${PROJECT_ID}/AMSU/AMSUAN15/L1C/2006/2006_08_01/NSS.AMAX.NK.D06213.S0110.E0255.B4271112.GC.L1C" > ${PROJECT_ID}/list_filea
;   $ echo "${PROJECT_ID}/AMSU/AMSUAN15/L1C/2006/2006_08_01/NSS.AMAX.NK.D06213.S0250.E0437.B4271213.GC.L1C" >> ${PROJECT_ID}/list_filea
;
; Using AMSU-B only ::
;
;    use_amsua=0
;    yyyy=2006
;    mm=8
;    dd=1
;    lonmin=-60.
;    lonmax=50.
;    latmin=-30.
;    latmax=45.
;    extract_amsuab, yyyy, mm, dd, use_amsua, lonmin, lonmax, latmin, latmax
;
;   ${PROJECT_ID}/AMSU/2006/08/amsub_20060801_060w30s_050e45n.dat must have been created.
;
; Using AMSU-A and AMSU-B ::
;
;    use_amsua=1
;    yyyy=2006
;    mm=8
;    dd=1
;    lonmin=-60.
;    lonmax=50.
;    latmin=-30.
;    latmax=45.
;    extract_amsuab, use_amsua, yyyy, mm, dd, lonmin, lonmax, latmin, latmax
;
;   ${PROJECT_ID}/AMSU/2006/08/amsua_amsub_20060801_060w30s_050e45n.dat must have been created.
;
; :History:
;
; $Id$
;
; $URL$
;
; - fplod 20120410
;
;   * build output directory if needed
;   * remove prompt from examples
;
; - lelod 20111121
;
;   * introduction du parametre "descending" (orbite descendante 1,
;     montante 0) dans le fichier de sortie : a tester!
;
; - fplod 20110818T160435Z aedon.locean-ipsl.upmc.fr (Darwin)
;
;   * no more /bdd in examples
;   * need to check if missing exist before NaN affection on idl7 cratos 
;
; - pinsard 2011-08-18T10:34:53Z loholt1.ipsl.polytechnique.fr (Linux)
;
;   * correction of examples
;   * examples with 2 files in list_filea and list_fileb
;   * match technique from http://cerebus.as.arizona.edu/~ioannis/teaching/idl/tutorial_strings.html
;   * revision du cas use_amsua=1 : la liste de fichiers amsu-a n'est ouverte qu'une seule fois
;
; - pinsard 2011-08-18T09:10:08Z loholt1.ipsl.polytechnique.fr (Linux)
;
;   * move bad values from -9999.99 to NaN
;
; - pinsard 2011-08-17T14:58:13Z loholt1.ipsl.polytechnique.fr (Linux)
;
;   * use geolocation_to_string_idl
;
; - pinsard 2011-08-10T10:46:37Z loholt1.ipsl.polytechnique.fr (Linux)
;
;   * remove blank in output filename
;   * revision of articifial example with use_amsua=0
;   * add articifial example with use_amsua=1 and try (but fail) to
;     make it work on loholt1
;   * remove length path dependencies using IDL function file_basename
;   * use same date and box of examples in correct_nadir_amsu.pro
;
; - fplod 20110810T085644Z aedon.locean-ipsl.upmc.fr (Darwin)
;
;   * revision of header
;
; - pinsard 2011-05-26T08:44:38Z loholt1.ipsl.polytechnique.fr (Linux)
;
;   * strmid(fich,51) is not anymore pertinent to isolate the correct information
;
;     It was ok when path as /bdd/AMSU-1C/AMSUAN15/L1C/2006/2006_07_28/::
;      c='/bdd/AMSU-1C/AMSUAN15/L1C/2006/2006_07_28/NSS.AMAX.NK.D06209.S1323.E1455.B4266162.GC.L1C'
;      b=strmid(c,51)
;      print, b
;      NK.D06209.S1323.E1455.B4266162.GC.L1C
;
;     but now path can be anything (PROJETC_ID environment variable)
;     so the correct way is to catch the last 36 characters
;
;     test sur skylla: fonctionne au moins pour un fichier!
;
; :Also:
;
;  called by :ref:`traite_amsuab.sh`
;
;  use :ref:`read_amsua1c.pro`, :ref:`read_amsub1c.pro`,
;  :func:`geolocation_to_string_idl`
;
;  next step :  :ref:`correct_nadir_amsu.pro`
;
; :Todo:
;
; changer le nom des fichiers de listes pour viter des conflits lors de runs
; simultans
;
; ajouter MHS
;
; eviter la creation d'un fichier vide (si pas de points dans la zone)
;
; tester la cohrence entre les fichiers prsents dans list_filea et list_fileb
; en terme de date avec yyyy, mm et dd donns en paramtre
;
; improve log
;
; get rid of uppercase
;
;
;-
PRO extract_amsuab, yyyy, mm, dd, use_amsua $
                  , lonmin, lonmax, latmin, latmax
;
compile_opt idl2, strictarrsubs
;
@cm_project

;===================================
; fichier de sortie comportant les observations amsu
;
;====================================

if (use_amsua eq 1) then begin
   prefix='amsua_amsub_'
endif else begin
   prefix='amsub_'
endelse
look = 'filename'
scale = 1.
geomin = geolocation_to_string_idl(lonmin, latmin, look, scale)
geomax = geolocation_to_string_idl(lonmax, latmax, look, scale)
        + string(mm,format='(I2.2)')   + '/'
;
fileout = project_id_env+ 'AMSU/' $
        + string(yyyy,format='(I4.4)') + '/' $
        + string(yyyy,format='(I4.4)') + '/' $
        + string(mm,format='(I2.2)')   + '/' $
        + prefix+string(yyyy,format='(I4.4)') $
        + string(mm,format='(I2.2)') $
        + string(dd,format='(I2.2)') + '_' $
        + geomin + '_' $
        + geomax $
        + '.dat'
;
; buid the directory if needed
dirout = file_dirname(fullfilename,/MARK_DIRECTORY)
IF (FILE_TEST(dirout, /DIRECTORY, /EXECUTABLE, /WRITE) EQ 0) THEN BEGIN
    FILE_MKDIR, dirout
ENDIF
;

print, 'ouverture pour ecriture de ', fileout
openw, lun_obs, fileout, /get_lun, ERROR = error
IF (error NE 0) then begin
   ras = report(['eee : can not open for writing '$
         + '!C' $
         + 'code : ' + !ERROR_STATE.MSG $
         + fileout])
   STOP
ENDIF


; ouverture des fichiers liste (annee, mois, jour, tous satellites) pour
; chaque instrument
; boucle sur les elements de la liste

; lecture de la liste des fichiers AMSU-A si necessaire
if (use_amsua EQ 1) THEN BEGIN
    list_filea = project_id_env+'list_filea'
    index_filea = 0
    nb_filea = file_lines(list_filea)
    a = STRARR(nb_filea)
    filea = STRARR(nb_filea)
    print, 'ouverture pour lecture de ', list_filea
    openr, lun_a, list_filea, /get_lun
    while (not eof(lun_a)) do begin
        onefile = ''
        readf, lun_a, onefile
        filea[index_filea] = onefile
        ; isolate string independant from satellite
        a[index_filea]=strmid(file_basename(filea[index_filea]),7)
        index_filea = index_filea + 1
    endwhile
endif

; lecture des fichiers de donnees
; AMSUB
list_fileb=project_id_env+'list_fileb'
print, 'ouverture pour lecture de ', list_fileb
openr, lun_b, list_fileb, /get_lun
fileb=''
while (not eof(lun_b)) do begin
    readf,lun_b,fileb
   ; isolate string independant from satellite
    b=strmid(file_basename(fileb),7)
   ; check for match between AMSU-A and AMSU-B
    if (use_amsua EQ 1) THEN BEGIN
        match = where(strmatch(a,b) eq 1, nmatch)
      ; if not only one match, sckip this file
        if (nmatch NE 1) then begin
            print, 'iii : ', string(nmatch) , 'match between ', a, ' and ' , b
            goto, labfile
        endif else begin
            print, 'iii : match position : ', string(match)
        endelse
    ENDIF
; AMSUA
    if (use_amsua EQ 1) THEN BEGIN
        COMMON amsua_header,ama_head
        COMMON amsua_data  ,ama_scan
        print, 'ouverture et lecture du fichier ', filea[match]
        openr,lu1,filea[match],Error=erra,/get_lun
        read_amsua1c,filea[match], flag1
        if (flag1 eq 0) then goto, labfile
        na=SIZE(reform(ama_scan.btemps[0,*,*]))
        na_scan=na[2]           ; nb lignes (fauchees) dans l'orbite
        na_fov=na[1]            ; nb points dans la fauchee
        satt_ide=ama_head.h_satid
        tt=REFORM(ama_scan.scnlintime/3600000.)
        nbpix_a = na_fov        ; nb pixels dans la fauchee AMSUA
    endif

    COMMON amsub_header,amb_head
    COMMON amsub_data  ,amb_scan
    print, 'lecture du fichier ', fileb
    read_amsub1c,fileb, flag2
    if (flag2 eq 0) then goto, labfile

    nb=SIZE(reform(amb_scan.btemps[0,*,*]))
    nb_scan=nb[2]               ; nb lignes (ou fauchees)
    nb_fov=nb[1]                ; nb pixels dans la fauchee
    nbpix_b = nb_fov            ; nb pixels dans la fauchee AMSUB
    nosat=amb_head.h_satid ; on le lit dans le fichier, donc pas besoin de l'info dans la liste
    tt=REFORM(amb_scan.scnlintime/3600000.)
    fovb=indgen(nbpix_b - 1 ) + 1
; dans le cas use_amsua=1 :
; selection des lignes amsub qui minimisent la distance avec le centre
; des lignes amsua
    if (use_amsua EQ 1 ) THEN BEGIN
        get_sync_pulse, use_amsua, pulse_line, descending
        bsync=where(pulse_line GT 0)
        async=pulse_line[bsync]-1 ;attention pb bornes, a revoir (-1 rajoute pour voir)
        sfo=where((fix(fovb+1) mod 3) eq 0)
        fov=fovb[sfo]
        nbpix= nbpix_a
        n=na
        n_scan=na_scan
        n_fov=na_fov
        naa_scan=n_elements(bsync)
    ENDIF ELSE BEGIN
        fov=fovb
        nbpix= nbpix_b
        n=nb
        n_scan=nb_scan
        n_fov=nb_fov
   ; on garde tous les points
    ENDELSE

    ntime_1=fltarr(nbpix,n_scan)
    fovy_1=fltarr(nbpix,n_scan)
    for i=0L, n_scan-1L do begin
        ntime_1[*,i]=tt[i]      ; le fichier amsub sert de reference
    endfor

    for j=0L, nbpix-1L do begin
        fovy_1[j,*]=j+1 ; on remplit un tableau de nscan lignes avec les valeurs de no de pixel
    endfor
;
; on lit la structure et on extrait les infos: temps, longitude,
; latitude, et les Tbs des differents canaux (15 pour AMSUA, 5 pour AMSUB)
    bmch1_1=REFORM(amb_scan.btemps[0,*,*]/100.)
    bmch2_1=REFORM(amb_scan.btemps[1,*,*]/100.)
    bmch3_1=REFORM(amb_scan.btemps[2,*,*]/100.)
    bmch4_1=REFORM(amb_scan.btemps[3,*,*]/100.)
    bmch5_1=REFORM(amb_scan.btemps[4,*,*]/100.)
    bmzen_1=REFORM(amb_scan.angles[0,*,*]/100.)
    bmlat_1=REFORM(amb_scan.latlon[0,*,*]/1.E4)
    bmlon_1=REFORM(amb_scan.latlon[1,*,*]/1.E4)
; recherche du sens de l'orbite : descendante (1) ou montante (0)
    lat0=bmlat_1[*,0]
    lat1=bmlat_1[*,1]
    if (mean(lat1-lat0) lt 0) then begin
        desc=1
    endif else begin
        desc=0
    endelse
;
;    si use_amsua=0 tout cela se reduit a b*=b*_1 et c'est tout
    if (use_amsua eq 0) then begin
 ;  bmzen=bmzen_1
 ;  bmlat=bmlat_1
 ;  bmlon=bmlon_1
        bmch1=bmch1_1
        bmch2=bmch2_1
        bmch3=bmch3_1
        bmch4=bmch4_1
        bmch5=bmch5_1
        amaday=REFORM(amb_scan.scnlindy)
        amatime=ntime_1
        amafov= fovy_1
        ntime=ntime_1
        jour_ama=amaday[0]
        amalon=bmlon_1
        amalat=bmlat_1
        amazen=bmzen_1
    endif else begin

; choix d'un pixel amsub sur 3 et une ligne synchronisee avec amsua
;
        bmzen_2=bmzen_1[fov,*]
        bmzen=bmzen_2[*,bsync]
        bmlat_2=bmlat_1[fov,*]
        bmlon_2=bmlon_1[fov,*]
        bmlat=bmlat_2[*,bsync]
        bmlon=bmlon_2[*,bsync]
        bmch1_2=bmch1_1[*,bsync]
        bmch1=bmch1_2[fov,*]
        bmch2_2=bmch2_1[*,bsync]
        bmch2=bmch2_2[fov,*]
        bmch3_2=bmch3_1[*,bsync]
        bmch3=bmch3_2[fov,*]
        bmch4_2=bmch4_1[*,bsync]
        bmch4=bmch4_2[fov,*]
        bmch5_2=bmch5_1[*,bsync]
        bmch5=bmch5_2[fov,*]

        amch1_1=REFORM(ama_scan.btemps[0,*,*]/100.)
        amch2_1=REFORM(ama_scan.btemps[1,*,*]/100.)
        amch3_1=REFORM(ama_scan.btemps[2,*,*]/100.)
        amch4_1=REFORM(ama_scan.btemps[3,*,*]/100.)
        amch5_1=REFORM(ama_scan.btemps[4,*,*]/100.)
        amch6_1=REFORM(ama_scan.btemps[5,*,*]/100.)
        amch7_1=REFORM(ama_scan.btemps[6,*,*]/100.)
        amch8_1=REFORM(ama_scan.btemps[7,*,*]/100.)
        amch9_1=REFORM(ama_scan.btemps[8,*,*]/100.)
        amch10_1=REFORM(ama_scan.btemps[9,*,*]/100.)
        amch11_1=REFORM(ama_scan.btemps[10,*,*]/100.)
        amch12_1=REFORM(ama_scan.btemps[11,*,*]/100.)
        amch13_1=REFORM(ama_scan.btemps[12,*,*]/100.)
        amch14_1=REFORM(ama_scan.btemps[13,*,*]/100.)
        amch15_1=REFORM(ama_scan.btemps[14,*,*]/100.)

        amch1=amch1_1[*,async]
        amch2=amch2_1[*,async]
        amch3=amch3_1[*,async]
        amch4=amch4_1[*,async]
        amch5=amch5_1[*,async]
        amch6=amch6_1[*,async]
        amch7=amch7_1[*,async]
        amch8=amch8_1[*,async]
        amch9=amch9_1[*,async]
        amch10=amch10_1[*,async]
        amch11=amch11_1[*,async]
        amch12=amch12_1[*,async]
        amch13=amch13_1[*,async]
        amch14=amch14_1[*,async]
        amch15=amch15_1[*,async]

        amzen_1=REFORM(ama_scan.angles[0,*,*]/100.)
        amazen=amzen_1[*,async]

        amlat_1=REFORM(ama_scan.latlon[0,*,*]/1.E4)
        amlon_1=REFORM(ama_scan.latlon[1,*,*]/1.E4)

        amalat=amlat_1[*,async]
        amalon=amlon_1[*,async]

        amtime_1=REFORM(ama_scan.scnlintime/3600000.)
        amday_1=REFORM(ama_scan.scnlindy)

     ; amatime=amtime_1(async)
        amaday=amday_1[async]
        amafov= fovy_1[*,async]
        amatime=ntime_1[*,async]
        jour_ama=amaday[0]      ; jour du passage
    ENDELSE
;-----------------
; ici on selectionne la zone definie
;-----------------
    zone=where((amalon ge lonmin) and (amalon le lonmax) $
             and (amalat ge latmin) and (amalat le latmax) )

    nn=n_elements(zone)
    if (nn GT 10) then begin
        flagg=1
        dam=fltarr(25,nn)
        dam[0,*]=amalon[zone]
        dam[1,*]=amalat[zone]
        dam[2,*]=amazen[zone]
        dam[3,*]=amafov[zone]
        dam[4,*]=amatime[zone]
        IF (use_amsua EQ 1) THEN BEGIN
            dam[5,*]=amch1[zone]
            dam[6,*]=amch2[zone]
            dam[7,*]=amch3[zone]
            dam[8,*]=amch4[zone]
            dam[9,*]=amch5[zone]
            dam[10,*]=amch6[zone]
            dam[11,*]=amch7[zone]
            dam[12,*]=amch8[zone]
            dam[13,*]=amch9[zone]
            dam[14,*]=amch10[zone]
            dam[15,*]=amch11[zone]
            dam[16,*]=amch12[zone]
            dam[17,*]=amch13[zone]
            dam[18,*]=amch14[zone]
            dam[19,*]=amch15[zone]
        ENDIF
        dam[20,*]=bmch1[zone]
        dam[21,*]=bmch2[zone]
        dam[22,*]=bmch3[zone]
        dam[23,*]=bmch4[zone]
        dam[24,*]=bmch5[zone]

      ; replace missing values -9999.99 by NaN
        index_missing = where(dam EQ -9999.99, nb_missing)
        if (nb_missing NE 0) THEN BEGIN
            dam[index_missing]=!values.f_nan
        ENDIF

      ; ecriture des donnees ligne par ligne dans le fichier resultant
        nab=nn
      ;
      ; mm : mois, a recuperer dans les parametres
      ; nosat: 15, 16 ou 17
        print,'orbite',desc
      for i=0L, nab-1L do begin
          
          if (use_amsua EQ 1) THEN BEGIN
;format: nosat,mois, day, lon, lat, zen,fov, time, ama_ch1:15,amb_ch1:5
 ; print, 'mois::::', mm,'-----', nab-i
              printf,lun_obs, mm, jour_ama,desc, dam[4,i], nosat, dam[3,i], dam[0,i], dam[1,i],$
                dam[2,i], dam[5,i], dam[6,i],dam[7,i],$
                dam[8,i], dam[9,i],dam[10,i], dam[11,i],dam[12,i],$
                dam[13,i], dam[14,i], dam[15,i], dam[16,i],dam[17,i],$
                dam[18,i], dam[19,i],$
                dam[20,i], dam[21,i],dam[22,i], dam[23,i],dam[24,i] ,$
                format='(3(i3,1x),f8.4,2i3,23(1x,f8.3))'
          ENDIF ELSE BEGIN
  ; format: orbit,lon, lat, zen,fov, time, day,mois,amb_ch1:5
              printf,lun_obs, mm, jour_ama,desc, dam[4,i], nosat, dam[3,i], dam[0,i], dam[1,i],$
                dam[2,i], dam[20,i], dam[21,i],dam[22,i], dam[23,i], dam[24,i], $
                format='(3(i3,1x),f8.4,2i3,8(1x,f8.3))'
          ENDELSE
      endfor
      
  endif else begin
      flagg=0
      print, 'pas de points dans la zone'
  endelse


; fin boucle sur les fichiers lus
labfile:
endwhile
print, 'fermeture de ', fileout
close, /all
end

