;+
;
; =============================
; geolocation_to_string_idl.pro
; =============================
;
; .. function:: geolocation_to_string_idl(lon_value, lat_value, look, scale)
;
; DESCRIPTION
; ===========
;
;    :param lon_value: longitude, W < 0
;    :units lon_value: deg
;    :type lon_value: double, long, int or float
;    :raise lon_value: required
;
;    :param lat_value: latitude, N > 0
;    :units lat_value: deg
;    :type lat_value: double, long, int or float
;    :raise lat_value: required
;
;    :param look: title or filename
;    :type look: string
;    :raise look: required
;
;    :param scale: scale factor to be applied to lat and long
;    :units scale: n.a.
;    :type scale: double
;    :raise scale: required
;
;    :returns: a string or -1 if error
;    :rtype: string
;
; Build a string using W, E, S or N according to values of latitude and 
; longitude
;
; latitude and  longitude will be multiply by ``scale``.
;
; if ``look`` is set to title and ``scale`` is set to 1., output will be
; yy.yyC,xxx.xxC.
;
; if ``look`` is set to filename  and ``scale`` is set to 1., output will be
; yyc,xxxc.
;
; EXAMPLES
; ========
;
; see demos
;
; SEE ALSO
; ========
;
; equivalent fonctionality :
; :func:`geolocation_to_string_sh.sh` (shell script)
; :func:`geolocation_to_string` (matlab/octave) when :samp:`look=filename`
; wil be implemented.
;
; TODO
; ====
;
; test if scale = 1, 10 or 100. 
;
; parameter convention ?
;
; parameter precision (nb de digit aprs la virgule dans la chaine)
; reconduct demo and test from octave
;
; EVOLUTIONS
; ==========
;
; $Id$
;
; $URL$
;
; - fplod 20111108T112023Z aedon.locean-ipsl.upmc.fr (Darwin)
;
;   * add scale parameter
;
; - pinsard 2011-08-18T08:48:36Z loholt1.ipsl.polytechnique.fr (Linux)
;
;   * oups : n/s for longitude ... 
;   * accept type long for lat_value and lon_value
;
; - pinsard 2011-08-18T07:14:31Z loholt1.ipsl.polytechnique.fr (Linux)
;
;   * reverse lat/lon order
;   * add reference to :ref:`geolocation_to_string_sh.sh`.
;
; - pinsard 2011-08-17T14:45:12Z loholt1.ipsl.polytechnique.fr (Linux)
;
;   * complete for look = filename
;
; - pinsard 2011-08-11T07:51:01Z loholt1.ipsl.polytechnique.fr (Linux)
;
;   * add look parameter "for title of figure" vs "for file name"
;   * creation from geolocation_to_string.m
;
;-
function geolocation_to_string_idl, lon_value, lat_value, look, scale
;
compile_opt idl2, strictarrsubs
@common
;
s = size(SCOPE_TRACEBACK(/STRUCTURE),/DIMENSION)
routine = (SCOPE_TRACEBACK(/STRUCTURE))[s-1].Routine
;
if key_performance EQ 1 THEN BEGIN
    time1 = SYSTIME(1)
ENDIF
;
; Return to caller if errors
ON_ERROR, 2
;
usage='geolocation_string = geolocation_to_string_idl(lon_value, lat_value, look, scale)';
;
geolocation_string=-1;
;
nparam = N_PARAMS()
IF (nparam LT 4) THEN BEGIN
    ras = report(['Incorrect number of arguments.' $
          + '!C' $
          + 'Usage : ' + usage])
    return, -1
ENDIF
;
arg_type=size(lon_value,/type);
if ((arg_type NE 2) AND (arg_type NE 3) AND (arg_type NE 4) AND (arg_type NE 5)) THEN BEGIN
    ras = report(['Incorrect arg type lon_value ' $
          + string(arg_type) $
          + '!C' $
          + 'Usage : ' + usage])
    return, -1
end
;
; +todo+ check only one lon_value
;
arg_type=size(lat_value,/type);
if ((arg_type NE 2) AND (arg_type NE 3) AND (arg_type NE 4) AND (arg_type NE 5)) THEN BEGIN
    ras = report(['Incorrect arg type lat_value ' $
          + string(arg_type) $
          + '!C' $
          + 'Usage : ' + usage])
    return, -1
end
;
; +todo+ check only one lat_value
;
if not finite(lon_value) THEN BEGIN
   msg = 'eee : lon_value NaN'
   ras = report(msg)
   return, -1
endif
;
if not finite(lat_value) THEN BEGIN
   msg = 'eee : lat_value NaN'
   ras = report(msg)
   return, -1
end
;
arg_type=size(look,/type);
if (arg_type NE 7) THEN BEGIN
    ras = report(['Incorrect arg type look ' $
          + string(arg_type) $
          + '!C' $
          + 'Usage : ' + usage])
    return, -1
end
CASE look OF
    'title' : BEGIN
         sep=',';
         unit_string='';
         IF (scale EQ 1) THEN BEGIN
            lon_format='(F6.2)'
            lat_format='(F5.2)'
         ENDIF ELSE IF (scale EQ 10) THEN BEGIN
            lon_format='(F7.1)'
            lat_format='(F6.1)'
         ENDIF ELSE IF (scale EQ 100) THEN BEGIN
            lon_format='(F8.0)'
            lat_format='(F7.0)'
         ENDIF ELSE BEGIN
                ras = report(['Incorrect value of scale' $
                      + string(scale) $
                      + '!C' $
                      + 'Allowed: 1, 10 or 100'])
                return, -1
         ENDELSE
         label_w = 'W'
         label_e = 'E'
         label_n = 'N'
         label_s = 'S'
    END
    'filename' : BEGIN
         sep='';
         unit_string='';
         IF (scale EQ 1) THEN BEGIN
            lon_format='(I3.3)'
            lat_format='(I2.2)'
         ENDIF ELSE IF (scale EQ 10) THEN BEGIN
            lon_format='(I4.4)'
            lat_format='(I3.3)'
         ENDIF ELSE IF (scale EQ 100) THEN BEGIN
            lon_format='(I5.5)'
            lat_format='(I4.4)'
         ENDIF ELSE BEGIN
                ras = report(['Incorrect value of scale' $
                      + string(scale) $
                      + '!C' $
                      + 'Allowed: 1, 10 or 100'])
                return, -1
         ENDELSE
         label_w = 'w'
         label_e = 'e'
         label_n = 'n'
         label_s = 's'
    END
    ELSE  : BEGIN
       msg = 'eee : invalid look parameter ' + look
       ras = report(msg)
       msg = 'eee : look must be one of title or filename'
       ras = report(msg)
       return, -1
    END
ENDCASE
;
arg_type=size(scale,/type);
if ((arg_type NE 2) AND (arg_type NE 3) AND (arg_type NE 4) AND (arg_type NE 5)) THEN BEGIN
    ras = report(['Incorrect arg type scale ' $
          + string(arg_type) $
          + '!C' $
          + 'Usage : ' + usage])
    return, -1
END
;
IF (lon_value > 0.) THEN BEGIN
   west_east=label_e;
ENDIF ELSE BEGIN
   west_east=label_w;
ENDELSE
IF (lat_value > 0.) THEN BEGIN
   south_north=label_n;
ENDIF ELSE BEGIN
   south_north=label_s;
ENDELSE
;
; conversion lon and lat absolute values to string
lon_string=strtrim(string(abs(lon_value*scale),format=lon_format),2);
lat_string=strtrim(string(abs(lat_value*scale),format=lat_format),2);
;
; build the result
geolocation_string = lon_string + unit_string + west_east $
                     + sep $
                     + lat_string + unit_string + south_north 
;
IF key_performance EQ 1 THEN BEGIN
   msg = report(['ppp : ' + routine + ' : dure totale ' $
               + string(SYSTIME(1)-time1,format='(F12.6)')])
ENDIF
;
return, geolocation_string
;
end

;!demo
;! lon_value=-10.
;! lat_value=10.
;! look='title'
;! scale = 1.
;! geolocation_string=geolocation_to_string_idl(lon_value,lat_value, look, scale)
;! print, geolocation_string

;!demo
;! lon_value=-10.
;! lat_value=10.
;! look='filename'
;! scale = 1.
;! geolocation_string=geolocation_to_string_idl(lon_value,lat_value, look, scale)
;! print, geolocation_string
;
;!test
;! lon_value=-10.
;! lat_value=10.
;! look='title'
;! scale = 1.
;! result='10.00W,10.00N'
;! assert(strcmp(geolocation_to_string_idl(lon_value,lat_value, look, scale),result))

;!test
;! lon_value=-10.
;! lat_value=10.
;! look='filename'
;! scale = 1.
;! result='010w10n'
;! assert(strcmp(geolocation_to_string_idl(lon_value,lat_value, look, scale),result))

;!test
;! lon_value=-10.12
;! lat_value=10.12
;! look='filename'
;! scale = 100.
;! result='01012w1012n'
;! assert(strcmp(geolocation_to_string_idl(lon_value,lat_value, look, scale),result))

;!test
;! lat_value=!values.f_nan
;! lon_value=-10.
;! look='title'
;! scale = 1.
;! fail('geolocation_to_string_idl(lon_value,lat_value, look, scale)', 'eee : geolocation_to_string_idl : lat_value NaN')

;!test
;! lon_value=!values.f_nan
;! lat_value=10.
;! look='title'
;! scale = 1.
;! fail('geolocation_to_string_idl(lon_value,lat_value, look, scale)', 'eee : geolocation_to_string : lon_value !values.f_nan')
