#! /bin/sh
#+
#
# .. program:: get_amsu.sh
#
# .. _get_amsu.sh:
#
# =======================================
# get_amsu.sh -- get AMSU reference files
# =======================================
#
# SYNOPSIS
# ========
#
# ::
#
#  $ get_amsu.sh [-f] -b yyyymmdd -e yyyymmdd
#
# DESCRIPTION
# ===========
#
# ``get_amsu.sh`` get AMSU reference files from **yyyymmddb** include to **yyyymmdde** excluded.
#
# .. option:: -f
# .. option:: -b beginning date <yyyymmdd>
# .. option:: -e end date <yyyymmdd>
#
# Log file is written on :file:`${PROJECT_LOG}/get_amsu.log.{YYYYMMDDTHHMMSSZ}`
#
# .. only:: man
#
#    Figure is visible on PDF and HTML documents only.
#
# .. only:: html or latex
#
#     .. graphviz::
#
#        digraph get_amsu {
#           graph [
#           rankdir="TB",
#           ]
#
#           amsu_ref_t1 [shape=diamond,fontname=Courier,label="loholt1.ipsl.polytechnique.fr:/bdd/AMMA/AOC-Paris/amsu/yyyy/mm/dd/prod_amsu01_yyyymmdd_hhmn_xxxxx.raw.gz"];
#
#           amsu_ref_t1 [shape=ellipse,fontname=Courier,label="/bdd/AMSU-1C/AMSU*/L1C/yyyy/yyyy_mm_dd/*.L1C"];
#
#
#           amsu_t1 [shape=ellipse,fontname=Courier,label="${PROJECT_ID}/AMSU/AMSU*/L1C/yyyy/yyyy_mm_dd/*.L1C"];
#
#
#           log [shape=ellipse,fontname=Courier,label="${PROJECT_LOG}/get_amsu.log.YYYYMMDDTHHMMSSZ"];
#
#           get_amsu [shape=box,
#           fontname=Courier,
#           color=blue,
#           URL="http://forge.ipsl.jussieu.fr/varamma/browser/trunk/src/get_amsu.sh",
#           label="${PROJECT}/src/get_amsu.sh"];
#
#           {amsu_ref_t1 amsu_ref_a_cor_land amsu_ref_a_cor_sea amsu_ref_b_cor_land amsu_ref_b_cor_sea} -> {get_amsu} -> {log amsu_t1 amsu_a_cor_land amsu_a_cor_sea amsu_b_cor_land amsu_b_cor_sea}
#
#          }
#
# EXAMPLES
# ========
#
# You don't have any AMSU reference data and you want data and correction file during [20060801, 20060802[, you just have to run this tool ::
#
#  $ get_amsu.sh -b 20060801 -e 20060802
#
# And look at log file with ::
#
#  $ tlogd.sh get_amsu
#
# and of course on files in ${PROJECT_ID}.
#
# CAUTIONS
# ========
#
# how to know if there is enough place before ?
#
# SEE ALSO
# ========
#
# :ref:`data_amsu`
#
# :ref:`varamma_profile.sh`
#
# Next step :
# :ref:`traite_amsuab.sh`
#
# TODO
# ====
#
# .. todo::
#
# make it work outside climserv
#
# exit on error if some correction files do not exist ?
# .. by the way how did correction files have been created ?
#
# check if NSS.AM?X.??.D?????.S????.E????.B???????.??.L1C ok
#
# what about
# lire_amsuab_new.pro:if (ambfile eq '/bdd/AMSU-1C/AMSUBN15/L1C/2006/2006_03_14/NSS.AMBX.NK.D06073.S1701.E1846.B4072728.GC.L1C') then goto,labett ;exception fichier erronne
# lire_amsuab_new.pro:if (ambfile eq '/bdd/AMSU-1C/AMSUBN15/L1C/2004/2004_06_25/NSS.AMBX.NK.D04177.S1824.E1950.B3180102.WI.L1C') then goto,labett ;exception fichier erronne
# build/complete catalog
#
# handle -f option
#
# describe next step traite_amsuab.sh
#
# EVOLUTIONS
# ==========
#
# $Id$
#
# $URL$
#
# - pinsard 20120529T150524Z loholt1.ipsl.polytechnique.fr (Linux)
#
#   * bug fix to handle correctly dates if translation in second on 9 digits
#     (trailing 0 is automaticaly interpreted as an octal number).
#     ex = 20010801
#   * bug fix to handle correctly dates if translation in second of begin
#     and end is not on the same number of digit
#     ex : -b 20010801 -e 20010931
#
# - pinsard 2011-05-05T09:05:55Z loholt1.ipsl.polytechnique.fr (Linux)
#
#   * creation
#
#-
system=$(uname)
case "${system}" in
   AIX|IRIX64)
      echo "www : no specific posix checking"
      date_cmd=date
   ;;
   Darwin)
      set -o posix
      date_cmd=gdate
   ;;
   Linux)
      set -o posix
      date_cmd=date
   ;;
   *)
     set -o posix
   ;;
esac
unset system
#
set -u
#
LANG=POSIX
#
command=$(basename ${0})
log_date=$(date -u +"%Y%m%dT%H%M%SZ")
#
usage=" Usage : ${command} [-f] -b yyyymmdd -e yyyymmdd"
#
hostname=$(hostname)
#
yyyymmddb_min=19991230
yyyymmdde_max=20120102
#
# default
force=0
yyyymmddb=20060801
yyyymmdde=20060802
#
minargcount=4
if [ ${#} -lt ${minargcount} ]
then
   echo "eee : not enought arguments"
   echo "${usage}"
   exit 1
fi
unset minargcount
#
while [ ${#} -gt 0 ]
do
   case ${1} in
      -b)
         # first date to get
         yyyymmddb=${2}
         shift
      ;;
      -e)
         # last date to get
         yyyymmdde=${2}
         shift
      ;;
      -f)
         force=1
      ;;
      -h)
         echo "${usage}"
         exit 0
      ;;
      *)
         # other choice
         echo "eee : unknown option ${1}"
         echo "${usage}"
         exit 1
      ;;
   esac
   # next flag
   shift
done
unset usage
#
# check parameters
# and define associated variables
#
${date_cmd} -u -d "${yyyymmddb}" > /dev/null
status_date=${?}
if [ ${status_date} -ne 0 ]
then
     echo "${command} : eee : yyyymmddb ${yyyymmddb} argument invalid"
     exit 1
fi
unset status_date
#
if [[ "${yyyymmddb}" < "${yyyymmddb_min}" ]]
then
     echo "${command} : eee : yyyymmddb ${yyyymmddb} must be equal or greater than yyyymmddb_min ${yyyymmddb_min}"
     exit 1
fi
#
${date_cmd} -u -d "${yyyymmdde}" > /dev/null
status_date=${?}
if [ ${status_date} -ne 0 ]
then
     echo "${command} : eee : yyyymmdde ${yyyymmdde} argument invalid"
     exit 1
fi
unset status_date
#
if [[ "${yyyymmdde}" > "${yyyymmdde_max}" ]]
then
     echo "${command} : eee : yyyymmdde ${yyyymmdde} must be lower or equal to yyyymmdde_max ${yyyymmdde_max}"
     exit 1
fi
#
if [[ "${yyyymmdde}" < "${yyyymmddb}" ]]
then
     echo "${command} : eee : yyyymmdde ${yyyymmdde} must be greater than yyyymmddb ${yyyymmddb}"
     exit 1
fi
#
if [[ "${yyyymmdde}" = "${yyyymmddb}" ]]
then
     echo "${command} : eee : yyyymmdde ${yyyymmdde} must be greater than yyyymmddb ${yyyymmddb}"
     exit 1
fi
#
# check for ${PROJECT_LOG} definition
if [ "${PROJECT_LOG}" = "" ]
then
   echo "${command} : eee : \${PROJECT_LOG} not defined"
   exit 1
fi
#
# check for ${PROJECT_LOG} existence
if [ ! -d ${PROJECT_LOG} ]
then
   echo "${command} : eee : ${PROJECT_LOG} not found"
   exit 1
fi
#
# check for permission access on PROJECT_LOG
if [ ! -x ${PROJECT_LOG} ]
then
   echo "${command} : eee : ${PROJECT_LOG} not reachable"
   exit 1
fi
#
# check for write permission on PROJECT_LOG
if [ ! -w ${PROJECT_LOG} ]
then
   echo "${command} : eee : ${PROJECT_LOG} not writable"
   exit 1
fi
#
log=${PROJECT_LOG}/$(basename ${0} .sh).log.${log_date}
echo "[Context]" 1>> ${log}
echo "command=$(basename ${0})" 1>>${log}
echo "hostname=${hostname}" 1>> ${log}
echo "runtime=${log_date}" 1>> ${log}
echo "log=${log}" 1>> ${log}
unset log_date
echo "" 1>> ${log}
echo "[Parameters]" 1>> ${log}
echo "yyyymmddb=${yyyymmddb}" 1>> ${log}
echo "yyyymmdde=${yyyymmdde}" 1>> ${log}
echo "" 1>> ${log}
#
#
list_url=""
#
# looking for data
dirref_base=/bdd/AMSU-1C/
#
# loop over all dates to build URL lists
current=$(${date_cmd} -u -d "${yyyymmddb}" +%s | awk '{printf "%d",$1}')
declare -i current
end=$(${date_cmd} -u -d "${yyyymmdde}" +%s | awk '{printf "%d",$1}')
declare -i end
declare -i delta=86400
while [ ${current} -lt ${end} ]
do
    echo "iii : looking for files of $(${date_cmd} -u -d "1970-01-01 ${current} sec" +%Y%m%dT%H%M)" 1>> ${log}
    yyyymmdd=$(${date_cmd} -u -d "1970-01-01 ${current} sec" +%Y%m%d)
    yyyy=$(${date_cmd} -u -d "1970-01-01 ${current} sec" +%Y)
    mm=$(${date_cmd} -u -d "1970-01-01 ${current} sec" +%m)
    dd=$(${date_cmd} -u -d "1970-01-01 ${current} sec" +%d)
    hhmn=$(${date_cmd} -u -d "1970-01-01 ${current} sec" +%H%M)
    for dirsat in ${dirref_base}/AMSU*
    do
        dirref="${dirsat}/L1C/${yyyy}/${yyyy}_${mm}_${dd}"
        if [ -d ${dirref}/ ]
        then
            #
            # look for these reference files
            bfile_t1=NSS.AM?X.??.D?????.S????.E????.B???????.??.L1C
            file_t1=${dirref}/${bfile_t1}
            find_file_t1=$(find ${dirref} -name ${bfile_t1} -type f)
            if [ "${find_file_t1}" = "" ]
            then
                echo "iii : ${file_t1} not found" >> ${log} 2>&1
            else
                echo "iii : ${file_t1} found" >> ${log} 2>&1
                list_url="${list_url} ${find_file_t1}"
            fi
        fi
    done
    unset yyyymmdd
    unset yyyy
    unset mm
    unset dd
    unset hhmn
    unset dirref
    #
    # next time step
    current=$((${current} + ${delta}))
done
unset current
unset end
#
if [ "${list_url}" = "" ]
then
   echo "iii : no files founds" >> ${log} 2>&1
else
    #
    # if file is not already in ${PROJECT_ID}, get it
    for url in ${list_url}
    do
	file=$(echo ${url} |  sed -e "s+${dirref_base}+${PROJECT_ID}/AMSU/+")
        if [ ! -f ${file} ]
        then
           loc=$(dirname ${file})
           if [ ! -d ${loc} ]
           then
              mkdir -p ${loc} 2> /dev/null
              echo "iii : adding directory ${loc}" >> ${log}
           fi
           ln -s ${url} ${file}
           echo "iii : adding file ${file}" >> ${log}
        else
           echo "iii : ${file} exists" >> ${log}
        fi
    done
    unset file
    unset url
fi
#

#
unset command
unset log
unset hostname
unset usage
#
unset date_cmd
#
# end
#++set
exit 0
