function [maxtab, mintab]=peakdet(v, delta, x)

%PEAKDET Detect peaks in a vector
%        [MAXTAB, MINTAB] = PEAKDET(V, DELTA) finds the local
%        maxima and minima ("peaks") in the vector V.
%        MAXTAB and MINTAB consists of two columns. Column 1
%        contains indices in V, and column 2 the found values.
%
%        With [MAXTAB, MINTAB] = PEAKDET(V, DELTA, X) the indices
%        in MAXTAB and MINTAB are replaced with the corresponding
%        X-values.
%
%        A point is considered a maximum peak if it has the maximal
%        value, and was preceded (to the left) by a value lower by
%        DELTA.
%
% Eli Billauer, 3.4.05 (Explicitly not copyrighted).
% This function is released to the public domain; Any use is allowed.

%+
%
% =========
% peakdet.m
% =========
%
% .. function:: peakdet(v, delta, x)
%
% DESCRIPTION
% ===========
%
%    :param v: vector where to find peaks
%    :type  v: 1D array of floats
%    :raise v: required
%
%    :param delta: minimal distance of two peaks
%    :type  delta: float
%    :raise delta: required
%
%    :param x: x coordinates of vectors v
%    :type  x: 1D array of floats, same dimension of v
%    :raise x: not required
%
%    :returns: maxima and minima if found
%              With [MAXTAB, MINTAB] = PEAKDET(V, DELTA, X) the indices
%              in MAXTAB and MINTAB are replaced with the corresponding
%              X-values.
%    :rtype: 1D array of floatsof maxima and 1D array vector of minima
%
% EXAMPLES
% ========
%
% see demo
%
% SEE ALSO
% ========
%
% :func:`plot_lai_gis`
%
% TODO
% ====
%
% test in VARAMMA project context or any signal
% see http://gil.69.free.fr/Matlab/Genesig/
%
% clean and comment demos
%
% EVOLUTIONS
% ==========
%
% $Id$
%
% $URL$
%
% - fplod 20111109T173133Z aedon.locean-ipsl.upmc.fr (Darwin)
%
%   * add more demo to learn usage
%
% - fplod 20111018T153329Z aedon.locean-ipsl.upmc.fr (Darwin)
%
%   * copy from http://www.billauer.co.il/peakdet.html
%
%-
maxtab = [];
mintab = [];
v = v(:); % Just in case this wasn't a proper vector

if nargin < 3
  x = (1:length(v))';
else
  x = x(:);
  if length(v)~= length(x)
    error('Input vectors v and x must have same length');
  end
end

if (length(delta(:)))>1
  error('Input argument DELTA must be a scalar');
end

if delta <= 0
  error('Input argument DELTA must be positive');
end

mn = Inf; mx = -Inf;
mnpos = NaN; mxpos = NaN;

lookformax = 1;

for i=1:length(v)
  this = v(i);
  if this > mx, mx = this; mxpos = x(i); end
  if this < mn, mn = this; mnpos = x(i); end

  if lookformax
    if this < mx-delta
      maxtab = [maxtab ; mxpos mx];
      mn = this; mnpos = x(i);
      lookformax = 0;
    end
  else
    if this > mn+delta
      mintab = [mintab ; mnpos mn];
      mx = this; mxpos = x(i);
      lookformax = 1;
    end
  end
end

%!demo
%! more off
%! t=0:0.001:10;
%! x=0.3*sin(t) + sin(1.3*t) + 0.9*sin(4.2*t) + 0.02*randn(1, 10001);
%! figure;
%! plot(x);
%! [maxtab, mintab] = peakdet(x, 0.5);
%! hold on;
%! plot(mintab(:,1), mintab(:,2), 'g*');
%! plot(maxtab(:,1), maxtab(:,2), 'r*');

%!demo
%! close all
%! t=0:0.001:10;
%! x=0.3*sin(t) + sin(1.3*t) + 0.9*sin(4.2*t) + 0.02*randn(1, 10001);
%! figure;
%! plot(t,x);
%! [maxtab, mintab] = peakdet(x, 0.5, t);
%! hold on;
%! plot(mintab(:,1), mintab(:,2), 'g*');
%! plot(maxtab(:,1), maxtab(:,2), 'r*');
%! disp([ '1er max a : ' num2str(maxtab(1,1))]);
%! disp([ 'value 1er max : ' num2str(maxtab(1,2))]);
%! t=maxtab(:,1)
%! x=maxtab(:,2)
%! plot(t,x);
%! [maxtab, mintab] = peakdet(x, 0.5, t);
%! hold on;
%! plot(mintab(:,1), mintab(:,2), 'g+');
%! plot(maxtab(:,1), maxtab(:,2), 'r+');


%!demo
%! % from http://www.developpez.net/forums/d940545/environnements-developpement/matlab/signal/detection-pics/
%! y=rand(1,15);
%! x=1:15;
%! sa=sign(diff([-inf y]));
%! sb=sign(diff([-inf y(end:-1:1)]));
%! sb=sb(end:-1:1);
%! idx=(sa==1 & sb==1);
%! figure
%! plot(x,y,'b-',x(idx),y(idx),'r*')

%!demo
%! %sans peakdet ++ marche pas car pas de singleton
%! x=1:52;
%! y=zeros(1,52);
%! y(15:40)=0.5;
%! y(20:35)=1;
%! y(25:27)=2.;
%! sa=sign(diff([-inf y]));
%! sb=sign(diff([-inf y(end:-1:1)]));
%! sb=sb(end:-1:1);
%! idx=(sa==1 & sb==1);
%! figure
%! plot(x,y,'b+',x(idx),y(idx),'r*')

%!demo
%! close all
%! %avec peakdet
%! x=1:7;
%! y=zeros(1,3);
%! y(1)=1;
%! y(2)=3;
%! y(3)=1;
%! y(5)=1;
%! y(6)=3;
%! y(7)=1;
%! plot(x,y)
%! [maxtab, mintab] = peakdet(y, 0.1, x);
%! whos
%! disp([ '1er max a : ' num2str(maxtab(1,1))]);
%! disp([ 'value 1er max : ' num2str(maxtab(1,2))]);
%! hold on;
%! plot(mintab(:,1), mintab(:,2), 'g*');
%! plot(maxtab(:,1), maxtab(:,2), 'r*');

%!demo
%! close all
%! %avec peakdet
%! x=1:3;
%! y=zeros(1,3);
%! y(1)=1;
%! y(2)=3;
%! y(3)=1;
%! plot(x,y)
%! [maxtab, mintab] = peakdet(y, 0.1, x);
%! whos
%! disp([ '1er max a : ' num2str(maxtab(1,1))]);
%! disp([ 'value 1er max : ' num2str(maxtab(1,2))]);
%! hold on;
%! plot(mintab(:,1), mintab(:,2), 'g*');
%! plot(maxtab(:,1), maxtab(:,2), 'r*');
