;+
;
; ====================
; search_amsufiles.pro
; ====================
;
; .. function:: search_amsufiles(numch, yyyy, mm, dd)
;
; DESCRIPTION
; ===========
;
; find files containing AMSU filenames on the given date in ${PROJECT_ID}
; :file:`${PROJECT_ID}/AMSU/AMSU${prefix}*/L1C/${yyyy}/${yyyy}_${mm}_${dd}/*.L1C`
;
; where prefix is the upper case of the first letter of **numch**
;
; This is the IDL equivalent of this shell command :
;
; .. code-block:: sh
;
;    prefix=$(printf  "%1c" ${numch} | tr [:lower:] [:upper:])
;    ls ${PROJECT_ID}/AMSU/AMSU${prefix}*/L1C/${yyyy}/${yyyy}_${mm}_${dd}/*.L1C
;
; SEE ALSO
; ========
;
; :ref:`extract_amsua.pro`
;
; EXAMPLES
; ========
;
; Using AMSU-A channel 5::
;
;   numch = 'a5'
;   yyyy=2006L
;   mm=8
;   dd=13
;   files_list = search_amsufiles(numch, yyyy, mm, dd)
;   print, files_list
;
; TODO
; ====
;
; check param
;
; tri chrono dans les list_file ou apres la production du fichier .dat
; par extract_amsua.pro avec sort -n : on a besoin de connaitre la
; convention de terminologie utilise.
;
; EVOLUTIONS
; ==========
;
; $Id$
;
; $URL$
;
; - fplod 20120419
;
;   * add some arg checking
;
; - fplod 20120323
;
;   * creation
;
;-
;
function search_amsufiles $
         , numch $
         , yyyy $
         , mm $
         , dd
;
compile_opt idl2, strictarrsubs, logical_predicate
;
@cm_project
@common
;
; Return to caller if errors
ON_ERROR, 2
;
result = -1
;
s = size(SCOPE_TRACEBACK(/STRUCTURE),/DIMENSION)
routine = (SCOPE_TRACEBACK(/STRUCTURE))[s-1].Routine
;
if key_performance EQ 1 THEN BEGIN
    time1 = SYSTIME(1)
ENDIF
;
usage = 'search_amsufiles(numch, yyyy, mm, dd)'
nparam = N_PARAMS()
IF (nparam NE 4) THEN BEGIN
   ras = report(['Incorrect number of arguments.' $
         + '!C' $
         + 'Usage : ' + usage])
   return, result
ENDIF
;
;
; check required and optional parameters
nparam = N_PARAMS()
IF (nparam LT 2) THEN BEGIN
   ras = report(['Incorrect number of arguments.' $
         + '!C' $
         + 'Usage : ' + usage])
   return, result
ENDIF
;
arg_type = size(yyyy,/type)
IF ((arg_type NE 2) AND (arg_type NE 3)) THEN BEGIN
   ras = report(['Incorrect arg type yyyy' $
         + '!C' $
         + 'Usage : ' + usage])
   return, result
ENDIF
;
arg_type = size(mm, /type)
IF ((arg_type NE 2) AND (arg_type NE 3)) THEN BEGIN
   ras = report(['Incorrect arg type mm' $
         + '!C' $
         + 'Usage : ' + usage])
   return, result
ENDIF
;
arg_type = size(dd, /type)
IF ((arg_type NE 2) AND (arg_type NE 3)) THEN BEGIN
   ras = report(['Incorrect arg type dd' $
         + '!C' $
         + 'Usage : ' + usage])
   return, result
ENDIF
;
yyyymmdd=jul2date(julday(mm,dd,yyyy))
IF ((yyyymmdd LT yyyymmddb_min) OR (yyyymmdd GT yyyymmdde_max)) THEN BEGIN
    msg = report([routine + 'eee : ' $
      + string(yyyy, format='(1x,I4.4)') $
      + string(mm, format='(I2.2)') $
      + string(dd, format='(I2.2)') $
      + ' must be between' $
      + string(yyyymmddb_min, format='(1x,I8.8)') $
      + ' and ' $
      + string(yyyymmdde_max, format='(1x,I8.8)') $
       ])
    return, result
ENDIF
;
; check for input directory
;
; test if ${PROJECT_ID} defined
CASE project_id_env OF
    ''  :  BEGIN
     msg = 'eee : ${PROJECT_ID} is not defined'
     ras = report(msg)
     return, result
           END
 ELSE: BEGIN
     msg = 'iii : ${PROJECT_ID} is ' + project_id_env
     ras = report(msg)
       END
ENDCASE
;
iodirin = isadirectory(project_id_env)
;
; existence and protection of ${PROJECT_ID}
IF (FILE_TEST(iodirin, /DIRECTORY, /EXECUTABLE, /READ) EQ 0) THEN BEGIN
   msg = 'eee : the directory' + iodirin  + ' is not accessible.'
   ras = report(msg)
   return, result
ENDIF
;
; build the files expression to be searched
nomcanal=strmid(numch,0,1)
nomcanal_upper = STRUPCASE(nomcanal)
;
files_expr = project_id_env $
           + 'AMSU/AMSU' + nomcanal_upper + '*/L1C/' $
           + string(yyyy,format='(i4.4)') + '/' $
           + string(yyyy,format='(i4.4)') + '_' $
           + string(mm,format='(i2.2)') + '_' $
           + string(dd,format='(i2.2)') +'/'+'*.L1C'
;
;print, files_expr
;
; search files
files_list = FILE_SEARCH(files_expr)
;
;print, files_list
;help, files_list
;print, size(files_list,/DIMENSION)
;help, size(files_list,/DIMENSION)

CASE size(files_list,/DIMENSION) OF
   0L : BEGIN
     print, routine, ': iii : pas de fichiers'
        END
   ELSE: BEGIN
       print, routine ,': iii : nb de fichiers', size(files_list,/DIMENSION)
        END
ENDCASE
;
result = files_list
;
IF key_performance EQ 1 THEN BEGIN
    msg = report(['ppp : ' + routine + ' : dure totale ' $
                   + string(SYSTIME(1)-time1,format='(F12.6)')])
ENDIF

return, result
;
end
