function [lon_value, lat_value, datejul_value, datestr_value, epsat_value]=simul_epsat(index_simulation)

%SIMUL_EPSAT fill array Surface Rainrate 3D array and associated arrays lat,
%lon, date with simulated data

%
%+
%
% =============
% simul_epsat.m
% =============
%
% .. function:: simul_epsat(index_simulation)
%
% DESCRIPTION
% ===========
%
%    :param index_simulation: index of simulation
%    :type index_simulation: uint8
%    :raise index_simulation: required
%
% Charge en mmoire un tableau 3D simul **epsat_value** de Surface Rainrate
% dimension 1 : temps, dimension 2 : lon,  dimension 3 : lat
%
% Simulation #1
%    la zone gographique [20E,10E] [10N,20N] avec un point tous les 5 en
%    latitude et 5 en longitude et dans la zone temporelle 20060726-20060727
%    avec un point toutes les 1/2 heure
%
% Simulation #2
%    la zone gographique [24.95E,4E] [24.95S,25N] avec 250 points en
%    latitude
%    et 500 points en longitude dans la zone temporelle 20060726-20060726
%    avec un point toutes les 1/2 heure
%
% Simulation #3
%    la zone gographique [20E,10E] [10N,20N] avec un point tous les 5 en
%    latitude et 5 en longitude et dans la zone temporelle 20060727
%
% Les longitudes sont stockes dans le tableau **lon_value**.
%
% Les latitudes sont stockes dans le tableau **lat_value**.
%
% Les dates au format `yyyymmddhhmn` sont stocks dans le tableau
% **datestr_value** et les dates en jour julien sont stockes dans
% **datejul_value**.
%
% EXAMPLES
% ========
%
% From scratch to create short array of Surface Rainrate and associated arrays
% lat, long and datestr:
%
% see demo 1
%
% To plot the grid::
%
%   octave> showgrid(lon_value, lat_value);
%
% From scratch to create long array of Surface Rainrate and associated arrays
% lat, long and datestr:
%
% see demo 2
%
% To plot the grid::
%
%   octave> result=showgrid(lon_value, lat_value);
%
% SEE ALSO
% ========
%
% :ref:`data_epsat`
%
% :func:`EPSATread`
%
% :func:`showgrid`
%
% :func:`cumulEPSAT_day`
%
% TODO
% ====
%
% des valeurs de Surface Rainrate plus ralistes y compris des NaN
%
% returned values when error
%
% use in other EPSAT tools
%
% EVOLUTIONS
% ==========
%
% $Id: simul_epsat.m 199 2011-03-04 10:11:00Z pinsard $
%
% $URL$
%
% - fplod 20110816T073113Z aedon.locean-ipsl.upmc.fr (Darwin)
%
%   * replace int8 by uint8 for integer parameters
%
% - fplod 20110609T080107Z aedon.locean-ipsl.upmc.fr (Darwin)
%
%   * add datejul_value output
%   * rearrange epsat_value dimension
%     dimension 1 : temps, dimension 2 : lon,  dimension 3 : lat
%
% - fplod 20110310T135704Z aedon.locean-ipsl.upmc.fr (Darwin)
%
%   * fix lon_delta for simulation 2
%   * correction on the time dimension of epsat_value (not
%     numel(datestr_value) )!
%   * correction to have 48 time steps / day (not 49 !)
%   * add simulation 3 for one day
%   * reduce simulation 2 to one day
%   * use linspace because strange behaviour of octave using lat_delta
%     (was 249 lat point instead of 250 in simulation #2)
%
%   * nb : no more crash on octave 3.2.4 cratos
%
% - fplod 20110308
%
%   * creation
%-
%
result=-1;
%
usage='lon_value, lat_value, datejul_value, datestr_value, epsat_value]=simul_epsat(index_simulation)';
if nargin~=nargin(mfilename)
   disp(['Incorrect number of input arguments ' num2str(nargin)]);
   error(usage)
end
%
if nargout ~= nargout(mfilename)
   disp(['Incorrect number of output arguments ' num2str(nargout)]);
   error(usage);
end
%
arg_info=whos('index_simulation');
if ~strcmp(arg_info.class,'uint8')
   disp(['Incorrect type of arg index_simulation']);
   whos index_simulation
   error(usage);
end
clear arg_info
%
% dfinition de la zone gographique : les coins en degr et les rsolutions
if (index_simulation == 1)
   lonmin_value=-25;
   lonmax_value=+25;
   latmin_value=-5.;
   latmax_value=20.;
   lat_delta = 5.;
   lon_delta = 5.;
   lat_nbpoint = NaN;
   lon_nbpoint = NaN;
elseif (index_simulation == 2 )
   % use first two and last values of LATITUDE and LONGITUDE of
   % surf-rr_epsat-sg_multi-sat_010d_30min_20060726_v3.1-02c.nc
   % to choose realistic geographic range
   lonmin_value=-24.95;
   lonmax_value=+25;
   latmin_value=-4.95;
   latmax_value=19.95;
   lat_delta = NaN;
   lon_delta = NaN;
   lat_nbpoint = 250;
   lon_nbpoint = 500;
elseif (index_simulation == 3)
   lonmin_value=-25;
   lonmax_value=+25;
   latmin_value=-5.
   latmax_value=20.
   lat_delta = 5.;
   lon_delta = 5.;
   lat_nbpoint = NaN;
   lon_nbpoint = NaN;
else
   error(['eee : unknow simulation ' num2str(index_simulation)]);
end
%
% dfinition de la zone temporelle : date dbut, date fin et la rsolution
if (index_simulation == 1)
   yyyymin_value=2006;
   yyyymax_value=2006;
   mmmin_value=07;
   mmmax_value=07;
   ddmin_value=26;
   ddmax_value=27;
   datestr_delta=(datenum(2000,1,2)-datenum(2000,1,1))/48.;
elseif (index_simulation == 2 )
   yyyymin_value=2006;
   yyyymax_value=2006;
   mmmin_value=07;
   mmmax_value=07;
   ddmin_value=26;
   ddmax_value=27;
   datestr_delta=(datenum(2000,1,2)-datenum(2000,1,1))/48.;
elseif (index_simulation == 3)
   yyyymin_value=2006;
   yyyymax_value=2006;
   mmmin_value=07;
   mmmax_value=07;
   ddmin_value=26;
   ddmax_value=27;
   datestr_delta=(datenum(2000,1,2)-datenum(2000,1,1))/48.;
else
   error(['eee : unknow simulation ' num2str(index_simulation)]);
end
clear index_simulation
%
% cration du tableau des longitudes
if (~isnan(lon_delta))
   lon_value=transpose(lonmin_value:lon_delta:lonmax_value);
   lon_nbpoint=size(lon_value,1);
   clear lon_delta
else
   lon_value=transpose(linspace(lonmin_value,lonmax_value,lon_nbpoint));
end
clear lonmin_value
clear lonmax_value
%
% cration du tableau des latitudes
if (~isnan(lat_delta))
   lat_value=transpose(latmin_value:lat_delta:latmax_value);
   lat_nbpoint=size(lat_value,1);
   clear lon_delta
else
   lat_value=transpose(linspace(latmin_value,latmax_value,lat_nbpoint));
end
clear latmin_value
clear latmax_value
clear lat_delta
%
datejul_value=datenum(yyyymin_value,mmmin_value,ddmin_value):datestr_delta:datenum(yyyymax_value,mmmax_value,ddmax_value);
clear yyyymin_value
clear yyyymax_value
clear mmmin_value
clear mmmax_value
clear ddmin_value
clear ddmax_value
clear datestr_delta
%
% conversion datenum (unit octave/matlab) en format "yyyymmddhhmn"
datestr_value=datestr(datejul_value,'yyyymmddHHMM');
datestr_value=datestr_value(1:end-1,:);
%
% cration de tableau Surface Rainrate
epsat_value=zeros(size(datestr_value,1),numel(lon_value),numel(lat_value));
epsat_value(:,:,:)=NaN;
for index_datestr=1:size(datestr_value,1)
    for index_lon=1:numel(lon_value)
        for index_lat=1:numel(lat_value)
            epsat_value(index_datestr,index_lon,index_lat)=(index_datestr+index_lon)/100. + (index_datestr+index_lat)/100.;
        end
   end
end
clear index_lat
clear numel(lon_value)
clear index_lon
clear index_datestr
%
% whos
%
result=0;
%
end

%!demo
%! % From scratch to create short array of Surface Rainrate and associated
%! % arrays lat, long and datestr::
%! varamma_startup
%! more off
%! index_simulation = uint8(1);
%! [lon_value, lat_value, datejul_value, datestr_value, epsat_value] = simul_epsat(index_simulation);
%! whos

%!demo
%! % From scratch to create long array of Surface Rainrate and associated
%! % arrays lat, long and datestr::
%! varamma_startup
%! more off
%! index_simulation = uint8(2);
%! [lon_value, lat_value, datejul_value, datestr_value, epsat_value] = simul_epsat(index_simulation);
%! whos

%!demo
%! % From scratch to create short array of Surface Rainrate and associated
%! % arrays lat, long and datestr::
%! varamma_startup
%! more off
%! index_simulation = uint8(3);
%! [lon_value, lat_value, datejul_value, datestr_value, epsat_value] = simul_epsat(index_simulation);
%! whos
