#! /bin/sh
#+
#
# .. program:: traite_trmm.sh
#
# .. _traite_trmm.sh:
#
# ==============
# traite_trmm.sh
# ==============
#
# SYNOPSIS
# ========
#
# ::
#
#  $ traite_trmm.sh -b yyyymmdd -e yyyymmdd -lonmin lonmin -lonmax lonmax -latmin latmin -latmax latmax
#
# DESCRIPTION
# ===========
#
# .. option:: -b beginning date <yyyymmdd>
# .. option:: -e end date <yyyymmdd>
# .. option:: -lonmin lonmin
# .. option:: -lonmax lonmax
# .. option:: -latmin latmin
# .. option:: -latmax latmax
#
# ``traite_trmm.sh`` concatenate :file:`${PROJECT_ID}/TRMM/3B40/{yyyy}/{mm}/{dd}/3B40RT.yyyymmddhh.5.nc`
# in :file:`${PROJECT_ID}/TRMM/3B40/{yyyy}/{mm}//3B40RT.yyyymmdd.5.nc`
#
# +todo+
#
# Log file is written on :file:`${PROJECT_LOG}/traite_trmm.log.{YYYYMMDDTHHMMSSZ}`
#
# .. only:: man
#
#    Figure is visible on PDF and HTML documents only.
#
# .. only:: html or latex
#
#     .. graphviz::
#
#        digraph traite_trmm {
#
#           trmm_t1 [shape=ellipse,fontname=Courier,label="${PROJECT_ID}/TRMM/prod/yyyy/mm/dd/prodRT.yyyymmddhh.x.nc"];
#           trmm_t2 [shape=ellipse,fontname=Courier,label="${PROJECT_ID}/TRMM/prod/yyyy/mm/prodRT.yyyymmdd.x.nc"];
#           trmm_t3 [shape=ellipse,fontname=Courier,label="${PROJECT_ID}/TRMM/prod/yyyy/mm/prodRT.yyyymmdd_geomin_geomax.x.nc"];
#
#           log [shape=ellipse,fontname=Courier,label="${PROJECT_LOG}/traite_trmm.log{YYYYMMDDTHHMMSSZ}"];
#
#           traite_trmm [shape=box,
#           fontname=Courier,
#           color=blue,
#           URL="http://forge.ipsl.jussieu.fr/varamma/browser/trunk/src/traite_trmm.sh",
#           label="${PROJECT}/src/traite_trmm.sh"];
#
#           {trmm_t1} -> {traite_trmm} -> {log trmm_t2 trmm_t3}
#
#         }
#
# EXAMPLES
# ========
#
# You already used :ref:`get_trmm.sh` to get TRMM reference data
# during [20060801, 20060802[ and you want to produce daily NetCDF files::
#
#   $ traite_trmm.sh -b 20060801 -e 20060802 -lonmin -25 -lonmax 25 -latmin -5 -latmax 20
#
# And look at log file with ::
#
#  $ tlogd.sh traite_trmm
#
# and of course on files in ${PROJECT_ID}.
#
# SEE ALSO
# ========
#
# :ref:`data_precipitation_microwave`
#
# :ref:`varamma_profile.sh`
#
# Previous step : :ref:`get_trmm.sh`
#
# Use :
# :ref:`geolocation_to_string_sh.sh`
# NCO_
#
# .. _NCO: http://nco.sourceforge.net/
#
#
# TODO
# ====
#
# check orginal files
#
# very strange view of outputs using netcdf but
# reference files /bdd/TRMM/3B40/NCDF/yyyy/\*.nc look also strange to me (fplod) !!!
#
# modify global attributes
#
# build/complete catalog
#
# adjust yyyymmddb_min and yyyymmdde_max
#
# handle -f option
#
# hard coded product and version
#
# unset
#
# EVOLUTIONS
# ==========
#
# $Id$
#
# $URL$
#
# - pinsard 20120529T150524Z loholt1.ipsl.polytechnique.fr (Linux)
#
#   * bug fix to handle correctly dates if translation in second on 9 digits
#     (trailing 0 is automaticaly interpreted as an octal number).
#     ex = 20010801
#   * bug fix to handle correctly dates if translation in second of begin
#     and end is not on the same number of digit
#     ex : -b 20010801 -e 20010931
#
# - pinsard 2011-08-18T07:27:26Z loholt1.ipsl.polytechnique.fr (Linux)
#
#   * replace filelatlon.sh by geolocation_to_string_sh.sh
#
#     .. warning::
#        filenames will be modified
#
# - pinsard 2011-06-06T15:09:14Z loholt1.ipsl.polytechnique.fr (Linux)
#
#   * reduce lon lat domain
#
# - fplod 20110606T135934Z cratos.locean-ipsl.upmc.fr (Linux)
#
#   * creation from traite_amsuab.sh
#
#-
system=$(uname)
case "${system}" in
   AIX|IRIX64)
     echo "www : no specific posix checking"
     date_cmd=date
   ;;
   Darwin)
     set -o posix
     date_cmd=gdate
   ;;
   Linux)
     set -o posix
     date_cmd=date
   ;;
   *)
    set -o posix
   ;;
esac
unset system
#
set -u
#
LANG=POSIX
#
command=$(basename ${0})
log_date=$(date -u +"%Y%m%dT%H%M%SZ")
#
usage=" Usage : ${command} [-f] -b yyyymmdd -e yyyymmdd -lonmin lonmin -lonmax lonmax -latmin latmin -latmax latmax"
#
hostname=$(hostname)
#
yyyymmddb_min=20060420
yyyymmdde_max=20061103
#
# default
debug=0
force=0
product="3B40"
version=5
yyyymmddb=20060420
yyyymmdde=20060420
#
minargcount=4
if [ ${#} -lt ${minargcount} ]
then
   echo "eee : not enought arguments"
   echo "${usage}"
   exit 1
fi
unset minargcount
#
while [ ${#} -gt 0 ]
do
   case ${1} in
     -b)
        # first date to get
        yyyymmddb=${2}
        shift
     ;;
     -e)
        # last date to get
        yyyymmdde=${2}
        shift
     ;;
     -lonmin)
        lonmin=${2}
        lonminf=$(printf "%f" $lonmin)
        shift
     ;;
     -lonmax)
        lonmax=${2}
        lonmaxf=$(printf "%f" $lonmax)
        shift
     ;;
     -latmin)
        latmin=${2}
        latminf=$(printf "%f" $latmin)
        shift
     ;;
     -latmax)
        latmax=${2}
        latmaxf=$(printf "%f" $latmax)
        shift
     ;;
     -f)
        force=1
     ;;
     -h)
        echo "${usage}"
        exit 0
     ;;
     *)
        # other choice
        echo "eee : unknown option ${1}"
        echo "${usage}"
        exit 1
     ;;
   esac
   # next flag
   shift
done
unset usage
#
# check parameters
# and define associated variables
#
case ${product} in
    3B40)
        prefix="3B40RT."
    ;;
    *)
      echo "${command} : eee : product ${product} argument invalid"
      exit 1
    ;;
esac
#
${date_cmd} -d "${yyyymmddb}" > /dev/null
status_date=${?}
if [ ${status_date} -ne 0 ]
then
    echo "${command} : eee : yyyymmddb ${yyyymmddb} argument invalid"
    exit 1
fi
unset status_date
#
if [[ "${yyyymmddb}" < "${yyyymmddb_min}" ]]
then
    echo "${command} : eee : yyyymmddb ${yyyymmddb} must be equal or greater than yyyymmddb_min ${yyyymmddb_min}"
    exit 1
fi
#
${date_cmd} -d "${yyyymmdde}" > /dev/null
status_date=${?}
if [ ${status_date} -ne 0 ]
then
    echo "${command} : eee : yyyymmdde ${yyyymmdde} argument invalid"
    exit 1
fi
unset status_date
#
if [[ "${yyyymmdde}" > "${yyyymmdde_max}" ]]
then
    echo "${command} : eee : yyyymmdde ${yyyymmdde} must be lower or equal to yyyymmdde_max ${yyyymmdde_max}"
    exit 1
fi
#
if [[ "${yyyymmdde}" < "${yyyymmddb}" ]]
then
    echo "${command} : eee : yyyymmdde ${yyyymmdde} must be greater than yyyymmddb ${yyyymmddb}"
    exit 1
fi
#
if [[ "${yyyymmdde}" = "${yyyymmddb}" ]]
then
     echo "${command} : eee : yyyymmdde ${yyyymmdde} must be greater than yyyymmddb ${yyyymmddb}"
     exit 1
fi
#
tool=ncrcat
type ${tool} 1> /dev/null 2>&1
status=${?}
if [ ${status} -ne 0 ]
then
   echo "${command} : eee : tool ${tool} not found"
   exit 1
fi
unset status
unset tool
#
tool=ncks
type ${tool} 1> /dev/null 2>&1
status=${?}
if [ ${status} -ne 0 ]
then
   echo "${command} : eee : tool ${tool} not found"
   exit 1
fi
unset status
unset tool
#
# check for write permission on PROJECT_LOG
if [ ! -w ${PROJECT_LOG} ]
then
   echo "${command} : eee : ${PROJECT_LOG} not writable"
   exit 1
fi
#
log=${PROJECT_LOG}/$(basename ${0} .sh).log.${log_date}
echo "[Context]" 1>> ${log}
echo "command=$(basename ${0})" 1>>${log}
echo "hostname=${hostname}" 1>> ${log}
echo "runtime=${log_date}" 1>> ${log}
echo "log=${log}" 1>> ${log}
unset log_date
echo "" 1>> ${log}
echo "[Parameters]" 1>> ${log}
echo "yyyymmddb=${yyyymmddb}" 1>> ${log}
echo "yyyymmdde=${yyyymmdde}" 1>> ${log}
echo "lonmin=${lonmin}" 1>> ${log}
echo "lonmax=${lonmax}" 1>> ${log}
echo "latmin=${latmin}" 1>> ${log}
echo "latmax=${latmax}" 1>> ${log}
echo "" 1>> ${log}
#
# loop over all dates
# set current and end date
current=$(${date_cmd} -u -d "${yyyymmddb}" +%s | awk '{printf "%d",$1}')
declare -i current
end=$(${date_cmd} -u -d "${yyyymmdde}" +%s | awk '{printf "%d",$1}')
declare -i end
declare -i delta=86400
while [ ${current} -lt ${end} ]
do
    echo "iii : looking for files of $(${date_cmd} -u -d "1970-01-01 ${current} sec" +%Y%m%dT%H%M)" 1>> ${log}
    yyyymmdd=$(${date_cmd} -u -d "1970-01-01 ${current} sec" +%Y%m%d)
    yyyy=$(${date_cmd} -u -d "1970-01-01 ${current} sec" +%Y)
    mm=$(${date_cmd} -u -d "1970-01-01 ${current} sec" +%m)
    dd=$(${date_cmd} -u -d "1970-01-01 ${current} sec" +%d)
    #

    # concatenate yyyymmddhh NetCDF files
    list=${PROJECT_ID}/TRMM/${product}/${yyyy}/${mm}/${dd}/${prefix}${yyyy}${mm}${dd}??.${version}.nc
    fileout_t2=${PROJECT_ID}/TRMM/${product}/${yyyy}/${mm}/${prefix}${yyyy}${mm}${dd}.${version}.nc
    ncrcat -O ${list} ${fileout_t2} >> ${log} 2>&1
    status=${?}
    if [ ${status} -ne 0 ]
    then
       echo "eee : pb with ncrcat" 1>> ${log}
       for file in ${list}
       do
           ncdump -h ${file} >> ${log} 2>&1
       done
       exit 1
    else
       echo "iii : adding ${fileout_t2}" >> ${log} 2>&1
    fi
    #
    # Extract zone
    geomin=$(geolocation_to_string_sh.sh -lon ${lonmin} -lat ${latmin})
    status=${?}
    if [ ${status} -ne 0 ]
    then
       echo "eee : pb geomin with geolocation_to_string_sh.sh" 1>> ${log}
       exit 1
    fi
    geomax=$(geolocation_to_string_sh.sh -lon ${lonmax} -lat ${latmax})
    status=${?}
    if [ ${status} -ne 0 ]
    then
       echo "eee : pb geomax with geolocation_to_string_sh.sh" 1>> ${log}
       exit 1
    fi
    fileout_t3=${PROJECT_ID}/TRMM/${product}/${yyyy}/${mm}/${prefix}${yyyy}${mm}${dd}_${geomin}_${geomax}.${version}.nc
    unset boxs
    ncks -O -d latitude,${latminf},${latmaxf} -d longitude,${lonminf},${lonmaxf} ${fileout_t2} ${fileout_t3}
    status=${?}
    if [ ${status} -ne 0 ]
    then
        echo "eee : pb with ncks on ${fileout_t2}" >> ${log} 2>&1
        ncdump -h ${fileout_t2} >> ${log} 2>&1
        exit 1
    else
        echo "iii : adding ${fileout_t3}" >> ${log} 2>&1
    fi
    unset status
    unset dd

    # next time step
    current=$((${current} + ${delta}))
done
#
unset yyyy
unset mm
#
unset command
unset log
unset hostname
unset usage
#
unset date_cmd
#
# end
#++set
exit 0
