function [result]=write_zi(west_value, east_value, north_value, south_value, zi_code, zi_description)

%WRITE_ZI write zones of interest file :file:`${PROJECT_ID}/maps/varamma_zi_write_zi.csv`
%from zi associated arrays lat, lon, name, region++

%
%+
%
% ===============
% write_zi.m
% ===============
%
% .. function:: write_zi(west_value, east_value, north_value, south_value, zi_code, zi_description)
%
% DESCRIPTION
% ===========
%
%    :param west_value: west most longitudes
%    :type west_value: 1d array of doubles
%    :raise west_value: required
%
%    :param east_value: east most longitudes
%    :type east_value: 1d array of doubles
%    :raise east_value: required
%
%    :param north_value: north most latitudes
%    :type north_value: 1d array of doubles
%    :raise north_value: required
%
%    :param south_value: south most latitudes
%    :type south_value: 1d array of doubles
%    :raise south_value: required
%
%    :param zi_code: codes of the zone of interest
%    :type zi_code: 1D cell array of strings
%    :raise zi_code: required
%
%    :param zi_description: descriptions of the zone of interest
%    :type zi_description: 1D cell array of strings
%    :raise zi_description: required
%
% Write in a file :file:`${PROJECT_ID}/maps/varamma_zi_write_zi.csv`
%
% EXAMPLES
% ========
%
% From scratch to write some simulated zi data in
% :file:`${PROJECT_ID}/maps/varamma_zi_write_zi.csv`:
%
% see demo 1
%
% From real file :file:`${PROJECT_ID}/maps/varamma_zi.csv`:
%
% see demo 2
%
% SEE ALSO
% ========
%
% :ref:`zones`
%
% :func:`simul_zi`
% :func:`read_zi`
%
% TODO
% ====
%
% coding rules
%
% connexion with define_area
%
% EVOLUTIONS
% ==========
%
% $Id$
%
% $URL: svn+ssh://pinsard@forge.ipsl.jussieu.fr/ipsl/forge/projets/varamma/svn/trunk/src/write_zi.m $
%
% - fplod 20110816T084401Z aedon.locean-ipsl.upmc.fr (Darwin)
%
%   * handle IO error
%
% - fplod 20110629T082432Z aedon.locean-ipsl.upmc.fr (Darwin)
%
%   * creation
%
%-
%
global application;
global application_version;
%
global PROJECT_ID;
%
result=-1;
%
usage='result=write_zi(west_value, east_value, north_value, south_value, zi_code, zi_description)';
%
if nargin~=nargin(mfilename)
   disp(['Incorrect number of input arguments ' num2str(nargin)]);
   error(usage);
end
%
if nargout ~= nargout(mfilename)
   disp(['Incorrect number of output arguments ' num2str(nargout)]);
   error(usage);
end
%
arg_info=whos('west_value');
if ~strcmp(arg_info.class,'double')
   disp(['Incorrect type of arg west_value' arg_info.class]);
   whos west_value
   error(usage);
end
clear arg_info
%
if ~isvector(west_value)
   disp('arg west_value must be a vector');
   error(usage);
end
%
arg_info=whos('east_value');
if ~strcmp(arg_info.class,'double')
   disp(['Incorrect type of arg east_value ' arg_info.class]);
   whos east_value
   error(usage);
end
clear arg_info
%
if ~isvector(east_value)
   disp('arg east_value must be a vector');
   error(usage);
end
%
arg_info=whos('north_value');
if ~strcmp(arg_info.class,'double')
   disp(['Incorrect type of arg north_value' arg_info.class]);
   whos north_value
   error(usage);
end
clear arg_info
%
if ~isvector(north_value)
   disp('arg north_value must be a vector');
   error(usage);
end
%
arg_info=whos('south_value');
if ~strcmp(arg_info.class,'double')
   disp(['Incorrect type of arg south_value ' arg_info.class]);
   whos south_value
   error(usage);
end
clear arg_info
%
if ~isvector(south_value)
   disp('arg south_value must be a vector');
   error(usage);
end
%
clear arg_info
%
arg_info=whos('zi_code');
if ~strcmp(arg_info.class,'cell')
   disp(['Incorrect type of arg zi_code ' arg_info.class]);
   whos zi_code
   error(usage);
end
clear arg_info
%
% +todo+   disp('arg zi_code must be a cell array of strings');
%
%
arg_info=whos('zi_description');
if ~strcmp(arg_info.class,'cell')
   disp(['Incorrect type of arg zi_description ' arg_info.class]);
   whos zi_description
   error(usage);
end
clear arg_info
%
% +todo+   disp('arg zi_description must be a cell array of strings');
%
% check for size of nb_lat nb_lon nb_zi
nb_zi=size(zi_description,1);
%
switch application
  case {'matlab'}
     test_consistent=isequal(numel(west_value), numel(east_value), numel(north_value, south_value), numel(zi_code), numel(zi_description));
  case {'octave'}
     test_consistent=size_equal(west_value, east_value, north_value, south_value, zi_code, zi_description)
  otherwise
     error('unknown application running');
end
if (not(test_consistent))
   disp(['eee : inconsistent dimensions of west_value, east_value, north_value, south_value, zi_code and zi_description']);
   whos west_value
   whos east_value
   whos north_value
   whos south_value
   whos zi_code
   whos zi_description
   error(usage);
end
%
% build ASCII filename to be written
fullfilename=[PROJECT_ID '/maps/'  'varamma_zi' '_' mfilename '.csv'];
disp(['iii : opening for writing ', fullfilename]);
[fid,message]=fopen(fullfilename,'w');
if (fid == -1)
    disp(message)
    error([ fullfilename ' can not be written'])
end
clear fullfilename
%
% write header
header=['code,description,north,south,east,west'];
format_header=['%s\n'];
clear format_header
clear header
%
% write data
%
sep=',';
% convert cell to string if needed
switch application
  case {'matlab'}
     zi_description = char(zi_description);
     zi_code = char(zi_code);
  case {'octave'}
     switch application_version
        case {'3.0.2'}
           zi_description = zi_description;
           zi_code = zi_code;
        case {'3.2.3','3.2.4','3.4.0'}
           zi_description = char(zi_description);
           zi_code = char(zi_code);
        otherwise
           error(['unknown application version running ', application , ' ', application_version '\n']);
     end
  otherwise
     error('unknown application running');
end
format_code='%s';
format_descritption='"%s"';
format_lat='%-6.3f';
format_lon='%-7.3f';
% loop on zis to write each data line
for index_zi=1:nb_zi
    description_trim=strtrim(sprintf(format_descritption,zi_description(index_zi,:)));
    code_trim=strtrim(sprintf(format_code,zi_code(index_zi,:)));
    west_trim=strtrim(sprintf(format_lon,west_value(index_zi)));
    east_trim=strtrim(sprintf(format_lon,east_value(index_zi)));
    north_trim=strtrim(sprintf(format_lat,north_value(index_zi)));
    south_trim=strtrim(sprintf(format_lat,south_value(index_zi)));
    fprintf(fid, '%s%s%s%s%s%s%s%s%s%s%s\n' ...
                       , code_trim(:) ...
                       , sep ...
                       , description_trim(:) ...
                       , sep ...
                       , north_trim(:) ...
                       , sep ...
                       , south_trim(:)
                       , sep ...
                       , east_trim(:) ...
                       , sep ...
                       , west_trim(:));
end
clear format_data
clear sep
%
fclose(fid);
clear fid
%
%++whos;
result=0;
%
end

%!demo
%! % From scratch to write some simulated zi data in
%! % :file:`${PROJECT_ID}/maps/varamma_zi_write_zi.csv`::
%! varamma_startup
%! more off
%! index_simulation = uint8(1);
%! [west_value, east_value, north_value, south_value, zi_code, zi_description] = simul_zi(index_simulation);
%! result = write_zi(west_value, east_value, north_value, south_value, zi_code, zi_description);

%!demo
%! % From real file :file:`${PROJECT_ID}/maps/varamma_zi.csv`::
%! varamma_startup
%! more off
%! [west_value, east_value, north_value, south_value, zi_code, zi_description] = read_zi();
%! result = write_zi(west_value, east_value, north_value, south_value, zi_code, zi_description);
